#####  Fine-tuned ecological niche models unveil climatic suitability and association with vegetation groups for selected Chaetocnema species in South Africa (Coleoptera: Chrysomelidae) #####
###################################################### Francesco Cerasoli, Paola D'Alessandro & Maurizio Biondi ##############################################################################
                                                                
### Create a new R project (go to "File" -> "New Project" in the R console) and then set the root directory for the analyses to the one where the .Rproj file was created
library(rprojroot)
path_root<-rprojroot::find_rstudio_root_file()

### set a random seed to facilitate reproducibility
set.seed(403)

library(raster)
rasterOptions(maxmemory=1e+10,progress="window",timer=T)
library(rgdal)
library(tidyverse)

### Load a shapefile of the entire Africa; this shapefile was previously downloaded into the root directory (You may download it here: http://geoportal.icpac.net/layers/geonode%3Aafr_g2014_2013_0)
setwd(path_root)
Africa_poly_WGS84<-readOGR(dsn='.',layer="afr_g2014_2013_0")

### Extract from the loaded shapefile a polygon mask representing South Africa, Lesotho and Swaziland (hereafter grouped under the 'S_Afr' label)
S_Afr_CountryID<-which(Africa_poly_WGS84$ADM0_NAME=="South Africa"|Africa_poly_WGS84$ADM0_NAME=="Lesotho"|Africa_poly_WGS84$ADM0_NAME=="Swaziland",arr.ind=T)
S_Afr_poly_WGS84<-Africa_poly_WGS84[S_Afr_CountryID,]

### Crop to the S_Afr extent a Digital Elevation Model (DEM) raster at 30 arc-seconds resolution previously downloaded into the root directory from the Worldclim 2.1 database (You may download it here: https://www.worldclim.org/data/worldclim21.html)
S_Afr_DEM_30secs<-mask(crop(raster("wc2.1_30s_elev.tif"),extent(S_Afr_poly_WGS84)),S_Afr_poly_WGS84)
DEM_colors<-colorRampPalette(c("seagreen1","limegreen","darkgreen","saddlebrown","darkgoldenrod4","snow4","snow2"))(8)
DEM_breaks<-c(0,600,1200,1800,2400,3200,4000,5000)

### Create an extent object for the S_Afr region excluding Prince Edward Islands as they are located really southwards compared to continental South Africa and do not host any Chaetocnema occurrence locality
library(terra)
S_Afr_Ext<-terra::ext(xmin(extent(S_Afr_poly_WGS84)),xmax(extent(S_Afr_poly_WGS84)),-36.00,ymax(extent(S_Afr_poly_WGS84)))

### Crop the S_Afr polygon to the obtained extent mask
S_Afr_poly_WGS84<-S_Afr_poly_WGS84 %>% terra::vect(.) %>% terra::crop(.,S_Afr_Ext)

#### Load occurrence data in WGS84 geographic coordinate system for the target Chaetocnema species
Chaetocnema_TargetSp<-c("brincki","danielssoni","darwini","gahani","natalensis")

Chaetocnema_TargetSp_OccurrWGS84<-read.csv2(paste0(path_root,"/Chaetocnema_SouthAfrica_OccurrWGS84.csv"),header = T)  ### You will find the .csv file with occurrence coordinates for our target Chaetocnema species within the Supplementary Materials of the paper

#### Draw alphahull-based range estimates for the selected species ####
library(alphahull)
library(pbapply)
library(sf)

Chaetocnema_TargetSp_Range<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(ggplot2)
  
  ### Select occurrences of the target species
  Occurs<-Chaetocnema_TargetSp_OccurrWGS84 %>% filter(grepl(sp,Species))
  ### Convert the Df of occurrences to SpatialPointsDataFrame to feed the ashape function
  coordinates(Occurs)<-c("Long_WGS84","Lat_WGS84")
  proj4string(Occurs) <- CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
  
  ### Create alphahull-based range estimate at alpha = 2.5 (average between alpha = 2 recommended by IUCN and alpha = 3 recommended in Burgman and Fox (2003))
  EstimatedRange<-ashape(coordinates(Occurs)[,1],coordinates(Occurs)[,2],alpha=2.5)
  
  ### Map the obtained alphahull-based range estimate on the S_Afr polygon map overlaid by the DEM layer
  S_Afr_poly_sf<-st_as_sf(S_Afr_poly_WGS84)
  ## crop the DEM to the previously obtained S_Afr extent mask
  S_Afr_DEM<-mask(crop(S_Afr_DEM_30secs,extent(S_Afr_poly_sf)),S_Afr_poly_sf)
  S_Afr_DEM_Df<-as.data.frame(as(S_Afr_DEM,"SpatialPixelsDataFrame"))
  names(S_Afr_DEM_Df)<-c("Elevation","Long_WGS84","Lat_WGS84")
    
  RangeMap<-ggplot()+
    geom_sf(data=S_Afr_poly_sf)+
    geom_tile(data=S_Afr_DEM_Df,aes(x=Long_WGS84,y=Lat_WGS84,fill=Elevation,alpha=0.95))+
    scale_fill_gradientn(colours=DEM_colors,breaks=DEM_breaks,labels=DEM_breaks)+
    guides(alpha=F)+
    labs(fill='Elevation (m)',x='Longitude',y='Latitude')+
    geom_point(data=as.data.frame(Occurs), aes(x=Long_WGS84,y=Lat_WGS84),colour='black',size=1.8,shape=6,stroke=0.8,alpha=0.8)+
    geom_segment(data=as.data.frame(EstimatedRange$edges), aes(x = x1, y = y1, xend = x2, yend = y2), color="red3",lineend = 'round',size=0.7)+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12,family='sans'),legend.text=element_text(size=11,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans'),axis.text.y=element_text(size=11,family='sans'))
  
  ### Return the alphahull-based range estimate
  return(list(EstimatedRange,RangeMap))
})

names(Chaetocnema_TargetSp_Range)<-Chaetocnema_TargetSp

for(i in 1:length(Chaetocnema_TargetSp_Range)){
  names(Chaetocnema_TargetSp_Range[[i]])<-c("AlphaHull","RangeMap")}

### Export a figure with range maps of the target species arranged on a single panel
library(ggpubr)

Chaetocnema_SAfr_RangeMaps<-ggarrange(plotlist = lapply(Chaetocnema_TargetSp,function(sp){return(Chaetocnema_TargetSp_Range[[sp]][['RangeMap']])}),
                                      labels=c("(a)","(b)","(c)","(d)","(e)"),font.label=list(size=12,family='sans',color='black'),ncol=2,nrow=3,align='hv',common.legend = T,legend='right')

ggsave(plot=Chaetocnema_SAfr_RangeMaps,filename="Chaetocnema_SAfr_AlphaHullRange.bmp",device='bmp',units="cm",width=21,height=28,dpi=600,limitsize = F)

### Project occurrence records to avoid latitudinal biases in the subsequent model fitting process (see below) ####
Chaetocnema_TargetSp_ProjectedOccurr<-pblapply(Chaetocnema_TargetSp,function(sp){
  ### Select occurrences of the target species
  Occurs<-Chaetocnema_TargetSp_OccurrWGS84 %>% filter(grepl(sp,Species))
  names(Occurs)<-c('Species','Easting','Northing')
  ### Convert the Df of occurrences to SpatialPointsDataFrame, then project this latter
  coordinates(Occurs)<-c("Easting","Northing")
  proj4string(Occurs) <- CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
  Occurs_projected<-spTransform(Occurs,CRS=CRS("+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs"))
  return(Occurs_projected)})

names(Chaetocnema_TargetSp_ProjectedOccurr)<-Chaetocnema_TargetSp

#### Crop worldwide bioclimatic variables (bio1-bio19) downloaded from Worldclim 2.1 (You may download them here: https://www.worldclim.org/data/worldclim21.html), at 2.5 arc-minutes resolution, to S_Afr boundaries and project them to 'Africa sinusoidal' projected coordinate system ####
path_vars_world<-paste0(path_root,"\\WorldClim2.1_2.5min_Worldwide\\2.5_Bioclim\\")  ## set the path from which the downloaded worldwide raster layers should be loaded into the R sessio
Bioclim_Curr_world_files<-list.files(paste0(path_vars_world,"Current"),pattern=".tif$")

Bioclim_names<-paste0('bio_',seq(1:19))

Bioclim_Curr_SAfr<-stack(pblapply(Bioclim_names,function(bio){
  ## Load the rasters of worldwide bioclimatic variables at 2.5 arcmin resolution
  Bio_curr_world<-raster(paste0(path_vars_world,"Current\\",grep(paste0(bio,".tif$"),Bioclim_Curr_world_files,value=T)))
  
  ## Crop and mask the rasters to the S_Afr boundaries and then stack it to the Africa sinusoidal coordinate system
  S_Afr_poly_WGS84<-as_Spatial(st_as_sf(S_Afr_poly_WGS84))
  Bio_curr_africa_proj<-projectRaster(mask(crop(Bio_curr_world,extent(S_Afr_poly_WGS84)),S_Afr_poly_WGS84),crs='+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs')
  
  return(Bio_curr_africa_proj)
  }))

names(Bioclim_Curr_SAfr)<-Bioclim_names

#### Perform a VIF analysis to check multicollinearity among the nineteen bioclim variables across the South African region
library(usdm)
VIF_Bioclim_Curr_SAfr<-vifstep(Bioclim_Curr_SAfr,th=5) 
## "bio2", "bio3", "bio8", "bio9", "bio13", "bio15" and "bio19" are retained (VIF < 5) under current conditions, so retain them for ENMs' fitting
ModellingVars_SAfr<-c("bio_2","bio_3","bio_8","bio_9","bio_13","bio_15","bio_19")
Bioclim_Curr_Sub_SAfr<-Bioclim_Curr_SAfr[[ModellingVars_SAfr]]

#### Start the Ecological Niche Modelling phase ####
if(dir.exists(paste0(path_root,"/1st_Rev/ENMs"))==F){                ### create a specific folder to store Ecological Niche Models (ENMs) results
  dir.create(paste0(path_root,"/1st_Rev/ENMs"),recursive = T)}

path_models<-paste0(path_root,"/1st_Rev/ENMs/")

#### Set parallel computing options to speed up some of the subsequent analyses ####
library(future)
library(furrr)

### use a baseline of 3/5 of the available cores as workers for parallel processing if not otherwise specified
nWorkingProcesses<-ceiling((3/5)*availableCores())
### return an error when non-exportable objects are detected
options(future.globals.onReference = "error")

### Given the low sample size (< 15) available for some target species, Random Forests and Maxent algorithms could be more suited than regression-based methods (e.g. GLM, GAM), so they are selected for model fitting
ModellingAlgo<-c("Maxent","RF")   

#### To feed Random Forests with the required pseudo-absences (background points), we generate for each target species a buffer polygon of ~5 degrees around its presence points
#### Later, we will randomly draw the pseudo-absences within this buffer (i.e. geographical exclusion, Barbet-Massin et al. (2012) Selecting pseudo‐absences for species distribution models: how, where and how many?. Methods in ecology and evolution, 3(2), 327-338)
Chaetocnema_TargetSp_BgBuffer<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(rgdal)
  require(rgeos)
  
  S_Afr_poly_Projected<-as_Spatial(st_as_sf(terra::project(S_Afr_poly_WGS84,"+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs"))) ### project the S_Afr boundary polygon to 'Africa sinusoidal' projected coordinate system for buffering
  
  ## Create an internal buffer of 10 km around the occurrence points to avoid pseudoabsences falling within the same cell of a presence point (i.e. class-overlap)
  internalBuffer<-gBuffer(Chaetocnema_TargetSp_ProjectedOccurr[[sp]],width=10000)
  ## Create an external buffer of 600 km around the occurrence points
  externalBuffer<-gBuffer(Chaetocnema_TargetSp_ProjectedOccurr[[sp]],width=600000)
  
  clippedBuffer<-gIntersection(gDifference(externalBuffer,internalBuffer),S_Afr_poly_Projected)
  
  ## Convert the buffer polygon to 'SpatialPolygonsDataFrame'
  if(class(clippedBuffer)[1]=="SpatialCollections"){
    clippedBuffer<-as(clippedBuffer@polyobj,"SpatialPolygonsDataFrame")}else{
      clippedBuffer<-as(clippedBuffer,"SpatialPolygonsDataFrame")}
  
  return(clippedBuffer)
})

names(Chaetocnema_TargetSp_BgBuffer)<-Chaetocnema_TargetSp

#### To accurately estimate ENMs' predictive performance, accounting for posssible Spatial Autocorrelation (SAC) in occurrence data, we will use cross-validation (CV) relying on geographically-based train-test partitioning ####
### For species with <15 occurrences (i.e. brincki, danielssoni and gahani), we will use buffered leave-one-out CV with 50 km radius (i.e. ~10 pixels, see Roberts et al. (2017) Cross‐validation strategies for data with temporal, spatial, hierarchical, or phylogenetic structure. Ecography, 40(8), 913-92)
### For the other species we will use checkerboard spatial blocking; to choose appropriate block sizes for these latter species, we will estimate SAC range in model residuals
### SAC range will be graphically estimated by fitting 'full data' Maxent and RF models (10 replicates each) for each species, computing residuals from predictions on the training data and then drawing a correlogram from Moran's I computed on these residuals
### The selected block size will equal the estimated SAC range increased by 50 km, as Roberts et al. (2017) suggest block size should be greater than the estimated SAC range to properly estimate prediction error

Chaetocnema_TargetSp_OccurrNumb<-data.frame(Species=Chaetocnema_TargetSp,nOccurr=sapply(Chaetocnema_TargetSp_ProjectedOccurr,function(x)nrow(x@data)),row.names = NULL)

### Fit the initial HSMs (10 replicates for both maxent and RF) for darwini and natalensis ####

if(dir.exists(paste0(path_models,"Spatial_Autocorr"))==F){
  dir.create(paste0(path_models,"Spatial_Autocorr"),recursive = T)}

path_SAC<-paste0(path_models,"Spatial_Autocorr/")

### Create species-specific folders to store outputs from initial Maxent models
lapply(c("darwini","natalensis"),function(sp){
  if(dir.exists(paste0(path_SAC,"initMaxent/",sp))==F){
    dir.create(paste0(path_SAC,"initMaxent/",sp),recursive = T)}
})

Checkerboard_SpxAlgo<-expand.grid(c("darwini","natalensis"),ModellingAlgo)
names(Checkerboard_SpxAlgo)<-c("Species","Algorithm")

Fit_initENMs<-function(sp,algo){
  ### get the presence points of the considered species
  Pres_Df<-data.frame(PresAbs=1,Chaetocnema_TargetSp_ProjectedOccurr[[sp]]@coords)
  names(Pres_Df)<-c("PresAbs","Easting","Northing")
  
  ### create a list to store predictions and residuals of initial ENMs
  initENMs_singleReps<-list()
  
  if(algo=='RF'){
    require(randomForest)
    
    for(i in 1:10){
      ### randomly select 100 pseudoabsences within the 5-degrees buffer polygon
      Bg_Df<-data.frame(PresAbs=0,coordinates(spsample(Chaetocnema_TargetSp_BgBuffer[[sp]],n=1000,type='random')))
      names(Bg_Df)<-names(Pres_Df)
      
      PresBg_Env<-cbind(rbind(Pres_Df,Bg_Df),raster::extract(Bioclim_Curr_Sub_SAfr,rbind(Pres_Df[,c('Easting','Northing')],Bg_Df[,c('Easting','Northing')]))) %>% drop_na(.)
      
      ### Fit the Random Forest model and compute residuals on training data
      RF_init<-randomForest(x=PresBg_Env[,names(Bioclim_Curr_Sub_SAfr)],y=as.factor(PresBg_Env[,'PresAbs']),n.trees=500)
      RF_init_pred<-round(predict(RF_init,PresBg_Env[,names(Bioclim_Curr_Sub_SAfr)],type='prob')[,2],2)
      RF_init_summary<-data.frame(PresBg_Env[,1:3],Preds=RF_init_pred,Residuals=round(RF_init_pred-PresBg_Env[,'PresAbs'],2))
      
      initENMs_singleReps[[i]]<-RF_init_summary}
  }else{
    require(dismo)
    require(sf)
    
    for(i in 1:10){
      ### Randomly draw 10000 background points from 600km-wide buffer around occurrences without the inner buffer of 10km
      ### Indeed, class overlap in environmental space between presence and background points seems to be a problem for RF and less for Maxent (see Valavi et al. (2021) Modelling species presence‐only data with random forests. Ecography, 44(12), 1731-1742) 
      Occurrences_sf<-sf::st_as_sf(Chaetocnema_TargetSp_ProjectedOccurr[[sp]])
      SixK_Buff<-sf::st_buffer(Occurrences_sf, dist = 600000) %>% sf::st_union() %>% sf::st_sf()
      Buffered_Vars<-stack(mask(crop(Bioclim_Curr_Sub_SAfr,SixK_Buff),SixK_Buff))
      
      Bg_Df<-data.frame(PresAbs=0,dismo::randomPoints(Buffered_Vars,n=10000,p=Pres_Df[,c('Easting','Northing')]) %>% as.data.frame())
      names(Bg_Df)<-c("PresAbs","Easting","Northing")
      
      ### Fit the Maxent model
      Maxent_init<-dismo::maxent(x=Buffered_Vars,p=Pres_Df[,c('Easting','Northing')],a=Bg_Df[,c('Easting','Northing')],path=paste0(path_SAC,"initMaxent\\",sp,"\\Rep_",i),args=c("-P","-J"))
      
      ### Predict the obtained model on training data and compute residuals
      PresBg_Env<-cbind(rbind(Pres_Df,Bg_Df),raster::extract(Buffered_Vars,rbind(Pres_Df[,c('Easting','Northing')],Bg_Df[,c('Easting','Northing')]))) %>% drop_na(.)
      Maxent_init_pred<-round(predict(Maxent_init,PresBg_Env[,names(Buffered_Vars)]),2)
      Maxent_init_summary<-data.frame(PresBg_Env[,1:3],Preds=Maxent_init_pred,Residuals=round(Maxent_init_pred-PresBg_Env[,'PresAbs'],2))
      
      initENMs_singleReps[[i]]<-Maxent_init_summary}
  }
  
  ### get a dataframe storing predictions and residuals from all the 10 replicates
  initENMs_RepsTot_Preds<-lapply(1:10,function(indx){PredsDf<-data.frame(initENMs_singleReps[[indx]],Replicate=paste0("Rep_",indx))
  return(PredsDf)}) %>% do.call('rbind',.)
  
  row.names(initENMs_RepsTot_Preds)<-NULL
  
  return(initENMs_RepsTot_Preds)}

### Fit the initial models
Chaetocnema_TargetSp_initENMs<-pbmapply(Fit_initENMs,Checkerboard_SpxAlgo$Species,Checkerboard_SpxAlgo$Algorithm,SIMPLIFY=F)

names(Chaetocnema_TargetSp_initENMs)<-paste0(Checkerboard_SpxAlgo$Species,"_",Checkerboard_SpxAlgo$Algorithm)

### Draw correlograms based on Moran's I computed on residuals from the initial ENMs
### start parallel computing ('multisession' mode from 'future' package)
plan(multisession,gc=T,workers=nWorkingProcesses)
tic()
Chaetocnema_TargetSp_initENMs_Correlog<-future_map2(.x=Checkerboard_SpxAlgo$Species,.y=Checkerboard_SpxAlgo$Algorithm,.f=function(sp,algo){
  require(ncf)
  
  ### get the dataframe storing model residuals for the considered species * algorithm combination
  Preds_Res_Df<-Chaetocnema_TargetSp_initENMs[[paste0(sp,"_",algo)]] %>% mutate(Algo=rep(algo,nrow(.)))
  
  ### draw a correlogram for each of the 10 replicates
  Correlog_Res<-lapply(1:10,function(indx){
    ### Select the data on model residuals corresponding to each replicate
    Preds_Res_Df_byRepl<-Preds_Res_Df %>% mutate(Easting=Easting/1000,Northing=Northing/1000) %>% filter(Replicate== paste0("Rep_",indx)) %>% select(Easting,Northing,Residuals)
    
    ### Compute Moran's I in model residuals at increasing inter-point distances (25 km pace)
    Correlog<-ncf::correlog(Preds_Res_Df_byRepl$Easting,Preds_Res_Df_byRepl$Northing,Preds_Res_Df_byRepl$Residuals,resamp=30,increment=25)
    Correlog_Df<-data.frame(Algo=algo,Replicate=paste0("Rep_",indx),WithinClass_AvgDist=round(Correlog$mean.of.class,2),N_pairs=Correlog$n,Moran_I=round(Correlog$correlation,2),WithinClass_pvalue=round(Correlog$p,2))
    return(Correlog_Df)
  })%>% do.call('rbind',.) ### Extract a summary Df across the 10 replicates
  
  return(Correlog_Res)})
toc()

## Close multisession
plan(sequential)

names(Chaetocnema_TargetSp_initENMs_Correlog)<-names(Chaetocnema_TargetSp_initENMs)

### Produce correlograms across replicates for each species * algorithm combination
Chaetocnema_TargetSp_initENMs_Correlog_Plot<-lapply(c("darwini","natalensis"),function(sp){
  Correlog_Df<-Chaetocnema_TargetSp_initENMs_Correlog[grep(sp,names(Chaetocnema_TargetSp_initENMs_Correlog))] %>% do.call('rbind',.)
  Correlog_Plot<-ggplot(Correlog_Df,aes(x=WithinClass_AvgDist,y=Moran_I))+
    geom_line(aes(colour=Replicate),size=0.8)+
    geom_hline(yintercept = c(-0.1,0.1),colour='red3',size=0.8,linetype="dotdash")+
    xlim(0,3000)+
    scale_colour_viridis_d(alpha=0.6)+
    facet_wrap(~Algo,nrow=1,ncol=2)+
    labs(y="Moran's I", x="Distance (km)")+
    theme(panel.background=element_rect(fill="gray95"),panel.grid.major.x=element_line(colour="gray80",size=0.3),panel.grid.minor.x=element_line(colour="gray90",size=0.3),panel.grid.major.y=element_line(colour="gray80",size=0.3),panel.grid.minor.y=element_line(colour="gray90",size=0.3),legend.position='none',axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.1),axis.text.y=element_text(size=11,family='sans'))+
    scale_y_continuous(breaks=seq(-1,1,by=0.1),limits=c(-1,1))
  return(Correlog_Plot)}
)

names(Chaetocnema_TargetSp_initENMs_Correlog_Plot)<-c("darwini","natalensis")

### Export the obtained correlograms to the hard drive
setwd(paste0(path_models,"Spatial_Autocorr"))

for(sp in c("darwini","natalensis")){ggsave(filename=paste0(sp,"_CorrelogOnResiduals_ENMsInit.png"), plot = Chaetocnema_TargetSp_initENMs_Correlog_Plot[[sp]],device=png(type="cairo",units="cm",width=24,height=18,res=600),limitsize = F)}

while(dev.cur()>1)dev.off()

#### Based on the obtained correlograms, the species-specific Spatial Autocorrelation (SAC) range in 'full data' model residuals is generally lower for RF than for Maxent ####
#### Taking into account both algorithms, overall block size could be set to 250 km for both C. darwini and C. natalensis ####

#### Get Pres-Background SpatialPointsDataframes for each species * algorithm combination
Chaetocnema_TargetSp_Algo<-expand.grid(Species=Chaetocnema_TargetSp,Algo=ModellingAlgo)

Chaetocnema_TargetSp_PresBackground_SpDfs<-pbmapply(function(sp,algo){
  ### retrieve the projected presence points of the considered species
  Pres_Df<-data.frame(PresAbs=1,Chaetocnema_TargetSp_ProjectedOccurr[[sp]]@coords)
  names(Pres_Df)<-c("PresAbs","Easting","Northing")
  
  if(algo=='RF'){
    #### for RF fitting, randomly draw 1000 background points within the 10km-600km buffer polygon
    Bg_Df<-data.frame(PresAbs=0,coordinates(spsample(Chaetocnema_TargetSp_BgBuffer[[sp]],n=1000,type='random')))
    names(Bg_Df)<-names(Pres_Df)
    
    PresBg_Env<-cbind(rbind(Pres_Df,Bg_Df),raster::extract(Bioclim_Curr_Sub_SAfr,rbind(Pres_Df[,c('Easting','Northing')],Bg_Df[,c('Easting','Northing')]))) %>% drop_na(.)
    SpatialDf<-SpatialPointsDataFrame(data=PresBg_Env,coords=PresBg_Env[,c("Easting","Northing")],proj4string=CRS("+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs"))
  }else{
    require(dismo)
    require(sf)
    ### for Maxent fitting, randomly draw 10000 background points from a 600km-wide buffer around occurrences of the considered species without the inner 10km buffer
    Occurrences<-sf::st_as_sf(Chaetocnema_TargetSp_ProjectedOccurr[[sp]])
    SixK_Buff<-sf::st_buffer(Occurrences, dist = 600000) %>% sf::st_union() %>% sf::st_sf()
    ModVars_Rs_Buff<-stack(mask(crop(Bioclim_Curr_Sub_SAfr,SixK_Buff),SixK_Buff))
    Bg_Df<-data.frame(PresAbs=0,dismo::randomPoints(ModVars_Rs_Buff,n=10000,p=Pres_Df[,c('Easting','Northing')]) %>% as.data.frame())
    names(Bg_Df)<-names(Pres_Df)
    
    PresBg_Env<-cbind(rbind(Pres_Df,Bg_Df),raster::extract(Bioclim_Curr_Sub_SAfr,rbind(Pres_Df[,c('Easting','Northing')],Bg_Df[,c('Easting','Northing')]))) %>% drop_na(.)
    SpatialDf<-SpatialPointsDataFrame(data=PresBg_Env,coords=PresBg_Env[,c("Easting","Northing")],proj4string=CRS("+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs"))
  }
  
  return(SpatialDf)},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY = F)

names(Chaetocnema_TargetSp_PresBackground_SpDfs)<-paste0(Chaetocnema_TargetSp_Algo$Species,"_",Chaetocnema_TargetSp_Algo$Algo)

#### Create the spatial blocks for buffered leave-one-out (LOO) and Checkerboard CV ####
library(blockCV)
Chaetocnema_TargetSp_SpatialBlocking<-pbmapply(function(sp,algo){
 
  ### set the species-specific block size for training-test splitting
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    BlockSz<-50000 #### for species to which LOO CV will be applied, consider a buffer of 50km around the withheld presence point
  } else{BlockSz<-250000} ### for the remaining species checkerboard blocking will be performed using the block size chosen based on the previously estimated SAC range
  
  ### retrieve the Pres-Bg spatial dataframe 
  PresPAs_SpDf<-Chaetocnema_TargetSp_PresBackground_SpDfs[[paste0(sp,"_",algo)]]
  
  if(Chaetocnema_TargetSp_OccurrNumb %>% filter(Species==sp) %>% select(nOccurr)<=15){
    ### Use buffered LOO blocking for species with less than 15 presence points
    SpatialBlocking<-buffering(speciesData = PresPAs_SpDf,species = 'PresAbs', theRange = BlockSz, spDataType = "PB", addBG=T, progress=T)
  }else{
    #### Use checkerboard blocking for species with higher sample size to balance computational effort and proper estimation of prediction error
    SpatialBlocking<-spatialBlock(speciesData = PresPAs_SpDf,species = 'PresAbs',rasterLayer=Bioclim_Curr_Sub_SAfr$bio_9,theRange = BlockSz,selection='checkerboard',numLimit=50,iteration=10)}
  
  return(SpatialBlocking)},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY=F)

names(Chaetocnema_TargetSp_SpatialBlocking)<-names(Chaetocnema_TargetSp_PresBackground_SpDfs)

### Export, as an illustrative example, a map showing the checkerboard blocking arrangement obtained for C. darwini and C. natalensis, with the corresponding presence points overlaid
if(dir.exists(paste0(path_models,"SpatialBlocks"))==F){
  dir.create(paste0(path_models,"SpatialBlocks"),recursive = T)}

setwd(paste0(path_models,"/SpatialBlocks"))

pbmapply(function(sp,algo){
  PointsDf<-as.data.frame(Chaetocnema_TargetSp_PresBackground_SpDfs[[paste0(sp,"_",algo)]])
  SpBlocksToPlot<-Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_",algo)]]
  
  Gg_SpBlocks<-SpBlocksToPlot$plot+
    geom_point(data=PointsDf%>%filter(PresAbs==1),aes(x=Easting,y=Northing),colour='darkmagenta',alpha=0.75,size=1.5,shape=17)+
    geom_point(data=PointsDf%>%filter(PresAbs==0),aes(x=Easting,y=Northing),colour='gray50',alpha=0.75,size=1.5,shape=4)
  
  ggsave(filename=paste0(sp,'_CheckerBoard_',algo,'.png'),plot = Gg_SpBlocks,device=png(type='cairo',units='cm',width=24,height=18,res=600),limitsize = F)
},Chaetocnema_TargetSp_Algo$Species[c(3,5,8,10)],Chaetocnema_TargetSp_Algo$Algo[c(3,5,8,10)],SIMPLIFY = F)

while(dev.cur()>1)dev.off()

#### Use the ENMeval package version 2.0.1 to fit and select optimized Maxent models for each target species, with models evaluated on the LOO-deriving or checkerboard-deriving test data ####
OptimizeMaxent<-function(sp){
  require(ENMeval)
  
  ### retrieve presence-background data of the considered species
  Pres_Background_Df<-Chaetocnema_TargetSp_PresBackground_SpDfs[[paste0(sp,"_Maxent")]]@data
  
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    ### for species with < 15 occurrences, only linear, quadratic and hinge features will be considered to test alternative Maxent parameterizations
    ### the Regularization Multiplier values evaluated range from 1 to 3 with 0.5 bins
    TuningPars<-list(fc = c("L", "LQ","LQH"), rm = seq(1,3,0.5))
    
    ### do Maxent evaluation for each train-test LOO split
    MaxentEval_LOOreps<-lapply(1:nrow(Pres_Background_Df %>% filter(PresAbs==1)),function(indx){
      ### retrieve LOO train-test split for the considered replicate
      LOO_split<-Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_Maxent")]][['folds']][[indx]]
      
      ### Fit Maxent models with each alternative parameterization setting
      MaxEntEval<-ENMevaluate(occs=Pres_Background_Df %>% filter(PresAbs==1) %>% select(Easting,Northing),
                              envs=Bioclim_Curr_Sub_SAfr,
                              bg=Pres_Background_Df %>% filter(PresAbs==0) %>% select(Easting,Northing),
                              partitions="testing",
                              algorithm='maxnet',
                              occs.testing=Pres_Background_Df[LOO_split[[2]],] %>% select(Easting,Northing),
                              tune.args=TuningPars,
                              parallel=F,
                              taxon.name=paste0(sp,"_LOOsplit",indx),
                              doClamp=F, ## as we wanted to let the model extrapolate
                              updateProgress=T)
      
      ### Find the optimal Maxent model by looking at parameterization leading to AUC on test data >= 0.8 and minimum AUCdiff (this latter to find lowly overfitting models)
      OptMod<-MaxEntEval@results.partitions %>% filter(auc.val>=0.8) %>% slice_min(abs(auc.diff))
      
      return(list(MaxEntEval,OptMod))})
    
    return(MaxentEval_LOOreps)
    gc()
  }else{
    ### For species with >15 occurrences, add product and threshold features to those being considered for Maxent parameterization
    TuningPars<-list(fc = c("L","LQ","LQH","LQHPT"), rm = seq(1,3,0.5))
    ### retrieve checkerboard train-test splits
    CheckerBoard_split<-Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_Maxent")]][['foldID']]
    ### do Maxent evaluation for each of the two train-test checkerboard splits
    MaxEntEval<-ENMevaluate(occs=Pres_Background_Df %>% filter(PresAbs==1) %>% select(Easting,Northing),
                            envs=Bioclim_Curr_Sub_SAfr,
                            bg=Pres_Background_Df %>% filter(PresAbs==0) %>% select(Easting,Northing),
                            partitions="user",
                            algorithm='maxnet',
                            user.grp = list(occs.grp=CheckerBoard_split[1:nrow(Pres_Background_Df %>% filter(PresAbs==1))], bg.grp = CheckerBoard_split[(nrow(Pres_Background_Df %>% filter(PresAbs==1))+1):length(CheckerBoard_split)]),
                            tune.args=TuningPars,
                            parallel=F,
                            taxon.name=paste0(sp,"_CheckerBoard"),
                            doClamp=F, ## as we wanted to let the model extrapolate
                            updateProgress=T)
    
    ### Find the optimal Maxent model by looking at parameterization leading to AUC on test data >= 0.8 and minimum AUCdiff (this latter to find lowly overfitting models)
    OptMod_ByFold<-lapply(1:2,function(fld){
      OptPars<-MaxEntEval@results.partitions %>% filter(fold==fld,auc.val>=0.8) %>% slice_min(abs(auc.diff))
      return(OptPars)
    })
    return(list(MaxEntEval,OptMod_ByFold))
    gc()}
}

plan(multisession,gc=T,workers=nWorkingProcesses)
Chaetocnema_TargetSp_TunedMaxent<-future_map(.x=Chaetocnema_TargetSp,.f=OptimizeMaxent,.progress=T,.options=furrr_options(seed=T))

plan(sequential)

names(Chaetocnema_TargetSp_TunedMaxent)<-Chaetocnema_TargetSp

for(sp in Chaetocnema_TargetSp){
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    names(Chaetocnema_TargetSp_TunedMaxent[[sp]])<-paste0("LOOsplit_",1:length(Chaetocnema_TargetSp_TunedMaxent[[sp]]))
    for(i in 1:length(Chaetocnema_TargetSp_TunedMaxent[[sp]])){
      names(Chaetocnema_TargetSp_TunedMaxent[[sp]][[i]])<-c("Maxent_Eval","Maxent_OptPars")
    }
  }else{
    names(Chaetocnema_TargetSp_TunedMaxent[[sp]])<-c("Maxent_Eval","Maxent_OptPars")
  }
}

##### Predict the optimized Maxent models across the study region and weight predictions from the single train-test splits based on the obtained AUC on test data to derive weighted average and weighted standard deviation of model predictions ####
if(dir.exists(paste0(path_models,"PredictedSuitability"))==F){
  dir.create(paste0(path_models,"PredictedSuitability"),recursive = T)}

Chaetocnema_TargetSp_PredictTunedMaxent<-function(sp){
  require(ENMeval)
  
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    TargetSp_OptimizedMod<-lapply(1:length(Chaetocnema_TargetSp_TunedMaxent[[sp]]),function(indx){
      
      ### Retrieve the Maxent optimal parameterization settings for the considered LOO split
      OptimalParam<-Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_OptPars']]
      
      if(nrow(OptimalParam)==0){
        ### in case none of the fitted models exceeded the AUCtest >= 0.8 threshold, select as sub-optimal parameterization the one leading to the highest AUCtest
        SubOptimalParam<-Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@results.partitions %>% slice_max(auc.val)
        
        if(nrow(SubOptimalParam)==1){
          OptimizedMod<-SubOptimalParam %>% select(tune.args) %>% as.character(.)
        }else{OptimizedMod<-SubOptimalParam[round(runif(1,1,nrow(SubOptimalParam))),]%>% select(tune.args) %>% pull(.)}
        
        ### Get the AUC values corresponding to the  chosen (sub)optimal parameterization
        OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@results.partitions %>% filter(tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)
        
        }else if(nrow(OptimalParam)==1){
          OptimizedMod<-OptimalParam %>% select(tune.args) %>% pull(.)
          ### Get the AUC values corresponding to the single optimal parameterization
          OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_OptPars']] %>% filter(tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)
          }else{
            ### if more than a single parameterization setting led to ENM with AUCtest >= 0.8 and equally minimal AUCdiff, randomly choose among these 'equally optimal' parameterizations
            OptimizedMod<-OptimalParam[round(runif(1,1,nrow(OptimalParam))),] %>% select(tune.args) %>% pull(.)
            ### Get the AUC values corresponding to the chosen optimal parameterization
            OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_OptPars']] %>% filter(tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)}
      
      OptimizedMod_Df<-data.frame(TunedParam=OptimizedMod,AUCtest=OptimizedMod_AUC[1],AUCdiff=OptimizedMod_AUC[2],CVsplit=paste0("LOO_",indx))
      ### Predict the model fitted with the found optimal parameterization across the study extent
      OptimizedMod_Preds<-ENMeval::eval.predictions(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']])[[OptimizedMod]]
      
      return(list(OptimizedMod_Df,OptimizedMod_Preds))})
    
    }else{
      TargetSp_OptimizedMod<-lapply(1:2,function(fld){
        ### Retrieve the Maxent optimal parameterization settings for the considered Checkerboard split
        OptimalParam<-Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_OptPars']][[fld]] 
      
        if(nrow(OptimalParam)==0){
        ### in case none of the fitted models exceeded the AUCtest >= 0.8 threshold, select as sub-optimal parameterization the one leading to the highest AUCtest
        SubOptimalParam<-Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@results.partitions %>% filter(fold==fld) %>% slice_max(auc.val)
        
        if(nrow(SubOptimalParam)==1){
          OptimizedMod<-SubOptimalParam %>% select(tune.args) %>% pull(.)
          }else{OptimizedMod<-SubOptimalParam[round(runif(1,1,nrow(SubOptimalParam))),] %>% select(tune.args) %>% pull(.)}
        
        ### Get the AUC values corresponding to the chosen optimal parameterization
        OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@results.partitions %>% filter(fold==fld,tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)
        }else if(nrow(OptimalParam)==1){
          OptimizedMod<-OptimalParam %>% select(tune.args) %>% pull(.)
          ### Get the AUC values corresponding to the single optimal parameterization
          OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_OptPars']][[fld]] %>% filter(tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)
          }else{
            ### if more than a single parameterization setting led to ENM with AUCtest >= 0.8 and equally minimal AUCdiff, randomly choose among these 'equally optimal' parameterizations
            OptimizedMod<-OptimalParam[round(runif(1,1,nrow(OptimalParam))),] %>% select(tune.args) %>% pull(.)
            ### Get the AUC values corresponding to the chosen optimal parameterization
            OptimizedMod_AUC<-Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_OptPars']][[fld]] %>% filter(tune.args==OptimizedMod) %>% select(auc.val,auc.diff) %>% round(.,2)
            }
      
      OptimizedMod_Df<-data.frame(TunedParam=OptimizedMod,AUCtest=OptimizedMod_AUC[1],AUCdiff=OptimizedMod_AUC[2],CVsplit=paste0("Checkerboard_",fld))
      ### Predict the model fitted with the found optimal parameterization across Africa
      OptimizedMod_Preds<-ENMeval::eval.predictions(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']])[[OptimizedMod]]
      
      return(list(OptimizedMod_Df,OptimizedMod_Preds))})
      }
  
  TargetSp_Preds_Stack<-stack(lapply(1:length(TargetSp_OptimizedMod),function(indx){return(TargetSp_OptimizedMod[[indx]][[2]])}))
  ### For each Cv split, use the AUCtest value obtained by the selected ENM as weight
  TargetSp_Preds_Weights<-sapply(1:length(TargetSp_OptimizedMod),function(indx){return(TargetSp_OptimizedMod[[indx]][[1]][,2])})
  
  ### Compute AUC-weighted average suitability and write the corresponding raster file
  setwd(paste0(path_models,"/PredictedSuitability"))
  TargetSp_Preds_WeightedAvg<-raster::weighted.mean(TargetSp_Preds_Stack,TargetSp_Preds_Weights,na.rm=T)
  writeRaster(TargetSp_Preds_WeightedAvg,filename=paste0(sp,"_TunedMaxent_WeightedAvgPreds.tif"),format='GTiff',overwrite=T)
  
  ### Compute AUC-weighted standard deviation of suitability and write the corresponding raster file
  for(i in 1:nlayers(TargetSp_Preds_Stack)){
    if(i==1){WeightedSD_SS<-TargetSp_Preds_Weights[i]*((TargetSp_Preds_Stack[[i]]-TargetSp_Preds_WeightedAvg)^2)
    }else{WeightedSD_SS<-stack(WeightedSD_SS,TargetSp_Preds_Weights[i]*((TargetSp_Preds_Stack[[i]]-TargetSp_Preds_WeightedAvg)^2))}
  }
  WeightedSD_Numer<-sum(WeightedSD_SS)
  WeightedSD_Denom<-((length(TargetSp_Preds_Weights)-1)/length(TargetSp_Preds_Weights))*sum(TargetSp_Preds_Weights)
  TargetSp_Preds_WeightedSD<-sqrt(WeightedSD_Numer/WeightedSD_Denom)
  writeRaster(TargetSp_Preds_WeightedSD,filename=paste0(sp,"_TunedMaxent_WeightedStdDevPreds.tif"),format='GTiff',overwrite=T)
  
  ### Load the written raster files to get stable connections
  WeightedAvgPreds<-raster(paste0(sp,"_TunedMaxent_WeightedAvgPreds.tif"))
  WeightedStdDevPreds<-raster(paste0(sp,"_TunedMaxent_WeightedStdDevPreds.tif"))
  
  ### Get a summary dataframe storing chosen (sub)optimal parameterization and corresponding AUC values across the Cv splits
  OptimizedMod_Df<-lapply(1:length(TargetSp_OptimizedMod),function(indx){return(TargetSp_OptimizedMod[[indx]][[1]])}) %>% do.call('rbind',.) %>% as.data.frame(.)
  
  ### Finally extract weighted average and weighted std. dev. of model predictions, along with the summary dataframe
  return(list(WeightedAvgPreds,WeightedStdDevPreds,OptimizedMod_Df))
}

Chaetocnema_TargetSp_TunedMaxent_PredsCurr<-pblapply(Chaetocnema_TargetSp,Chaetocnema_TargetSp_PredictTunedMaxent)

names(Chaetocnema_TargetSp_TunedMaxent_PredsCurr)<-Chaetocnema_TargetSp

for(i in 1:length(Chaetocnema_TargetSp_TunedMaxent_PredsCurr)){
  names(Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[i]])<-c("WeightedAvg","WeightedStdDev","AUCstats")
}

### Use the randomization procedure implemented in the fitMaxnet package to calculate variables importance scores for the 'optimal' Maxent model selected through ENMeval for each species * train-test split combination
library(devtools)
devtools::install_github("peterbat1/fitMaxnet")
library(fitMaxnet)

Chaetocnema_TargetSp_TunedMaxent_VarImp<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(ENMeval)
  require(fitMaxnet)
  
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    TargetSp_LOO_VarImp<-lapply(1:length(Chaetocnema_TargetSp_TunedMaxent[[sp]]),function(indx){
      ### Retrieve the Maxent parameters chosen for the considered LOO split
      TunedParam<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][['AUCstats']][indx,"TunedParam"]
      ### Retrieve the model fitted with the chosen parameterization
      OptimizedMod<-ENMeval::eval.models(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']])[[TunedParam]]
      
      ### Retrieve row numbers of the single presence point and neighboring background points withheld for model validation in the considered LOO split
      TestData_RowNumb<-as.numeric(rownames(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@occs.testing))
      
      ### Convert the full set of presence and background data to the SWD format required to run the "varImportance" function from the "fitMaxnet" package
      FullData_SWD<-data.frame(PresAbs=c(rep(1,nrow(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@occs)),rep(0,nrow(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@bg))),rbind(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@occs,Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']]@bg))
      
      ### Retrieve training data and calculate standardized variables importance scores
      TrainData_SWD<-FullData_SWD[-TestData_RowNumb,]
      TrainData_SWD_occs<-TrainData_SWD %>% filter(PresAbs==1)
      TrainData_SWD_bg<-TrainData_SWD %>% filter(PresAbs==0)
      VarImp_df<-data.frame(Std_Imp=round(fitMaxnet::varImportance(theModel=OptimizedMod,occSWD=TrainData_SWD_occs,bkgSWD=TrainData_SWD_bg,responseType='cloglog',numReplicates=3),2))
      return(VarImp_df)
      }) %>% do.call('cbind',.)
    
    colnames(TargetSp_LOO_VarImp)<-paste0("LOOsplit",1:length(Chaetocnema_TargetSp_TunedMaxent[[sp]]),"_StdImp")
    return(TargetSp_LOO_VarImp)
    
    }else{
      TargetSp_Checkerboard_VarImp<-lapply(1:2,function(fld){
        ### Retrieve the Maxent parameters chosen for the considered Checkerboard split
        TunedParam<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][['AUCstats']][fld,"TunedParam"]
        ### Retrieve the model fitted with the chosen parameterization
        OptimizedMod<-ENMeval::eval.models(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']])[[TunedParam]]
        
        ### Retrieve row numbers of presence and background points falling in the training checkerboard fold
        TestData_RowNumb<-as.numeric(which(!Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_Maxent")]][['foldID']]==fld))
        
        ### Convert the full set of presence and background data to the SWD format required to run the "varImportance" function from the "fitMaxnet" package
        FullData_SWD<-data.frame(PresAbs=c(rep(1,nrow(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@occs)),rep(0,nrow(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@bg))),rbind(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@occs,Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']]@bg))
        
        ### Retrieve training data and calculate standardized importance score for each modelling variable
        TrainData_SWD<-FullData_SWD[-TestData_RowNumb,]
        TrainData_SWD_occs<-TrainData_SWD %>% filter(PresAbs==1)
        TrainData_SWD_bg<-TrainData_SWD %>% filter(PresAbs==0)
        VarImp_df<-data.frame(Std_Imp=round(fitMaxnet::varImportance(theModel=OptimizedMod,occSWD=TrainData_SWD_occs,bkgSWD=TrainData_SWD_bg,responseType='cloglog',numReplicates=3),2))
        return(VarImp_df)
        }) %>% do.call('cbind',.) 
    
    colnames(TargetSp_Checkerboard_VarImp)<-paste0("Checkerboard",1:2,"_StdImp")
    return(TargetSp_Checkerboard_VarImp)
  }
})

names(Chaetocnema_TargetSp_TunedMaxent_VarImp)<-Chaetocnema_TargetSp

## remove no more useful object(s)
rm(Chaetocnema_TargetSp_initENMs)
gc()

##### Fit Random Forests (RF) models with a number of different parameterizations for each species * train-test split combination and then find the optimized model for the considered combination #####
OptimizeRF<-function(sp){
  require(randomForest)
  require(PresenceAbsence)
  
  ### Retrieve presence-background data and spatial blocking folds of the considered species
  Pres_Background_Df<-Chaetocnema_TargetSp_PresBackground_SpDfs[[paste0(sp,"_RF")]]@data
  SpatBlocks<-Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_RF")]]
  
  nFolds<-length(SpatBlocks[['folds']])
  
  RF_OptMods<-lapply(1:nFolds,function(fld){
    ### Retrieve training and test data resulting from LOO- or Checkerboard-based splitting
    TrainData<-Pres_Background_Df[SpatBlocks[['folds']][[fld]][[1]],]
    TestData<-Pres_Background_Df[SpatBlocks[['folds']][[fld]][[2]],]
    
    ### Set the different RF parameterizations to test
    RF_parameters<-expand.grid(numTrees=c(500,1000,2000),NodeSize=c(1,5,ceiling(0.25*nrow(TrainData %>% filter(PresAbs==1))))) %>% unique(.)
  
    ### Fit RF model on train data for each parameterization, predict on the corresponding test data, then compute AUC on these latter (AUCtest), as well as AUC on the training data themselves to later calculate AUCdiff
    RF_AlternativeMods<-mapply(function(Trees,NdSz){
      ### convert the PresAbs column to factor to fit classification RF models
      TrainData[,'PresAbs']<-as.factor(TrainData[,'PresAbs'])
      TrainPresNb<-nrow(TrainData %>% filter(PresAbs==1)) ### get the number of training presence records to implement the "Down-sample RF" models (Valavi et al., 2021 Modelling species presence-only data with random forests. Ecography, 44(12)) 
      TrainAbsNb<-nrow(TrainData %>% filter(PresAbs==0)) ### get also the number of training pseudoabsences 
      if(TrainPresNb>TrainAbsNb){
        SmplSize=c("0" = TrainAbsNb, "1" = TrainAbsNb)}else{  ### set the sample size for each classification tree of the forest to the less abundant class in the training dataset
          SmplSize=c("0" = TrainPresNb, "1" = TrainPresNb)
        }
      
      RF_mod<-randomForest(x=TrainData[,names(Bioclim_Curr_Sub_SAfr)],y=TrainData[,'PresAbs'],ntree=Trees,nodesize=NdSz,sampsize=SmplSize,importance=T)
      
      ### Create a Df with Pres-PAs from test data and the corresponding RF predictions to compute AUCtest
      RF_preds_test<-round(predict(RF_mod,TestData[,names(Bioclim_Curr_Sub_SAfr)],type='prob')[,2],2)
      RF_TestObsPreds<-data.frame(ID=paste0('Site_',seq(1:nrow(TestData))),Test_Obs=TestData[,'PresAbs'],Test_Preds=RF_preds_test)
      RF_auc_test<-PresenceAbsence::auc(RF_TestObsPreds,st.dev=F,na.rm=T)
      
      ### Create a Df with Pres-PAs from train data and the corresponding RF predictions to compute AUC_train
      RF_preds_train<-round(predict(RF_mod,TrainData[,names(Bioclim_Curr_Sub_SAfr)],type='prob')[,2],2)
      RF_TrainObsPreds<-data.frame(ID=paste0('Site_',seq(1:length(TrainData[,'PresAbs']))),Train_Obs=as.numeric(as.character(TrainData[,'PresAbs'])),Train_Preds=RF_preds_train) ### training PresAbs data are converted back to numeric for AUC calculation
      RF_auc_train<-PresenceAbsence::auc(RF_TrainObsPreds,st.dev=F,na.rm=T)
      
      ### Compute AUC_diff
      RF_auc_diff<-RF_auc_train-RF_auc_test
      return(list(RF_mod,RF_AUC_stats=data.frame(Ntrees=Trees,NodeSz=NdSz,AUCtest=RF_auc_test,AUCdiff=RF_auc_diff)))
    },RF_parameters$numTrees,RF_parameters$NodeSize,SIMPLIFY=F)
    
    ### find the RF parameterization leading to AUCtest >= 0.8 and minimum AUCdiff
    RF_AlternativeMods_DiscriminationPerf<-lapply(RF_AlternativeMods,function(ls){
      AUCperf<-ls[['RF_AUC_stats']]
      return(AUCperf)}) %>% do.call('rbind',.) %>% mutate(ParameterIndx=1:length(RF_AlternativeMods))
    
    OptimalParam<-RF_AlternativeMods_DiscriminationPerf %>% filter(AUCtest>=0.8) %>% slice_min(abs(AUCdiff)) %>% as.data.frame(.)
    
    if(nrow(OptimalParam)==0){
      ### In case no model parameterization leads to AUCtest >= 0.8, select the one leading to the maximum AUCtest
      SubOptimalParam<-RF_AlternativeMods_DiscriminationPerf %>% slice_max(AUCtest)%>% as.data.frame(.)
      
      if(nrow(SubOptimalParam)==1){
        ### get RF model and AUC statistics from the found 'sub-optimal' parameterization
        OptimizedMod<-RF_AlternativeMods[[SubOptimalParam[1,"ParameterIndx"]]]
        }else{
          ### get RF model and AUC statistics from a randomly chosen 'equally sub-optimal' parameterization
          OptimizedMod<-RF_AlternativeMods[[SubOptimalParam[round(runif(1,1,nrow(SubOptimalParam))),'ParameterIndx']]]
        }
    
      }else if(nrow(OptimalParam)==1){
        ### get RF model and AUC statistics from the found optimal parameterization
        OptimizedMod<-RF_AlternativeMods[[OptimalParam[1,"ParameterIndx"]]]
        }else{
          ### if more than a single parameterization led to ENM with AUCtest >= 0.8 and equally minimal AUCdiff, randomly choose among these 'equally optimal' parameterizations
          OptimizedMod<-RF_AlternativeMods[[OptimalParam[round(runif(1,1,nrow(OptimalParam))),"ParameterIndx"]]]
        }
    
    names(OptimizedMod)<-c("RF_OptMod","AUCstats")
    return(OptimizedMod)})
  
  ### Return the list containing the selected RF model and the corresponding parameters and AUC statistics for each train-test split
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    names(RF_OptMods)<-paste0("LOO_",1:nFolds)
    }else{names(RF_OptMods)<-paste0("Checkerboard_",1:nFolds)}
  
  return(RF_OptMods)}
  
plan(multisession,gc=T,workers=nWorkingProcesses)
Chaetocnema_TargetSp_TunedRF<-future_map(.x=Chaetocnema_TargetSp,.f=OptimizeRF,.progress=T,.options=furrr_options(seed=T))

plan(sequential)  

names(Chaetocnema_TargetSp_TunedRF)<-Chaetocnema_TargetSp   
  
##### Predict the optimized RF models across the study region and weight predictions from the single train-test splits based on the obtained AUC on test data to derive weighted average and weighted standard deviation of model predictions ####
Chaetocnema_TargetSp_PredictTunedRF<-function(sp){
  require(randomForest)
  
  ### Convert the stack of the selected bioclimatic variables to a data frame to match the requirements of the 'predict' function from 'randomForest'
  ModVarsDf<-as.data.frame(as(Bioclim_Curr_Sub_SAfr,"SpatialPixelsDataFrame"))
  
  ### Predict the optimized RF model found for each train-test split to the study area extent, then compute weighted average and weighted standard deviation of such predictions across the splits based on the corresponding AUCtest values
  RF_ModLs<-Chaetocnema_TargetSp_TunedRF[[sp]]
  
  ### create a stack from RF predictions from each train-test split
  Preds_BySplit<-stack(lapply(1:length(RF_ModLs),function(indx){
    RFOptMod_BySplit<-RF_ModLs[[indx]][["RF_OptMod"]]
    Preds_bySplit<-predict(RFOptMod_BySplit,newdata=ModVarsDf,type='prob')[,2]
    Preds_bySplit_Df<-cbind(Preds_bySplit,ModVarsDf[,c("x","y")])
    coordinates(Preds_bySplit_Df)=~x+y
    gridded(Preds_bySplit_Df)=T
    projection(Preds_bySplit_Df)<-projection(Bioclim_Curr_Sub_SAfr)
    Preds_bySplit_Rs<-raster(Preds_bySplit_Df)
    return(Preds_bySplit_Rs)
  }))  
  
  ### create a vector of weights (i.e AUCtest from the different train-test split)
  Weights_BySplit<-sapply(1:length(RF_ModLs),function(indx){return(RF_ModLs[[indx]][["AUCstats"]][,"AUCtest"])})
  
  ### Compute AUC-weighted average suitability across the train-test splits
  WeightedAvg_Preds<-raster::weighted.mean(Preds_BySplit,Weights_BySplit,na.rm=T)  
  
  ### Compute AUC-weighted standard deviation of suitability
  for(i in 1:nlayers(Preds_BySplit)){
    if(i==1){WeightedSD_SS<-Weights_BySplit[i]*((Preds_BySplit[[i]]-WeightedAvg_Preds)^2)
    }else{WeightedSD_SS<-stack(WeightedSD_SS,Weights_BySplit[i]*((Preds_BySplit[[i]]-WeightedAvg_Preds)^2))}
  }
  WeightedSD_Numer<-sum(WeightedSD_SS)
  WeightedSD_Denom<-((length(Weights_BySplit)-1)/length(Weights_BySplit))*sum(Weights_BySplit)
  WeightedStdDev_Preds<-sqrt(WeightedSD_Numer/WeightedSD_Denom)  
  
  ### resample the obtained rasters of weighted predictions to the stack of the selected bioclimatic variables to be sure they also match dimensions of the Maxent-based weighted predictions
  WeightedAvg_Preds_Rsmpl<-raster::resample(WeightedAvg_Preds,Bioclim_Curr_Sub_SAfr)
  WeightedStdDev_Preds_Rsmpl<-raster::resample(WeightedStdDev_Preds,Bioclim_Curr_Sub_SAfr)
  
  ### write the resampled rasters to the hard drive
  setwd(paste0(path_models,"/PredictedSuitability"))
  
  writeRaster(WeightedAvg_Preds_Rsmpl,filename=paste0(sp,"_TunedRF_WeightedAvgPreds.tif"),format='GTiff',overwrite=T)
  writeRaster(WeightedStdDev_Preds_Rsmpl,filename=paste0(sp,"_TunedRF_WeightedStdDevPreds.tif"),format='GTiff',overwrite=T)
  
  ### Load the written raster files to get stable connections
  WeightedAvg_Rs<-raster(paste0(sp,"_TunedRF_WeightedAvgPreds.tif"))
  WeightedStd_Rs<-raster(paste0(sp,"_TunedRF_WeightedStdDevPreds.tif"))
  
  ### Get the rasters of weighted mean and weighted SD of predictions, and the vector of weights (i.e. AUC_test) from the single train-test splits
  return(list(WeightedAvg_Rs,WeightedStd_Rs))
}

Chaetocnema_TargetSp_TunedRF_PredsCurr<-pblapply(Chaetocnema_TargetSp,Chaetocnema_TargetSp_PredictTunedRF)

names(Chaetocnema_TargetSp_TunedRF_PredsCurr)<-Chaetocnema_TargetSp

for(i in 1:length(Chaetocnema_TargetSp_TunedRF_PredsCurr)){
  names(Chaetocnema_TargetSp_TunedRF_PredsCurr[[i]])<-c("WeightedAvg","WeightedStdDev")
}

#### For each target species, extract standardized variables importance scores from the optimized RF model found for each train-test split ####
Chaetocnema_TargetSp_TunedRF_VarImp<-pblapply(Chaetocnema_TargetSp,function(sp){
  RF_OptMods_Ls<-Chaetocnema_TargetSp_TunedRF[[sp]]
  ### extract variables importance scores for each of the train-test splits
  RF_VarImp_BySplit<-lapply(1:length(RF_OptMods_Ls),function(indx){
    RF_mod<-RF_OptMods_Ls[[indx]][["RF_OptMod"]]
    RF_VarImp<-data.frame(Std_Imp=round((RF_mod$importance[,'MeanDecreaseGini']/sum(RF_mod$importance[,'MeanDecreaseGini'])*100),1))
    rownames(RF_VarImp)<-row.names(RF_mod$importance)
    return(RF_VarImp)
  }) %>% do.call('cbind',.) 
  
  colnames(RF_VarImp_BySplit)<-paste0(names(RF_OptMods_Ls),"_StdImp")
  
  return(RF_VarImp_BySplit)})

names(Chaetocnema_TargetSp_TunedRF_VarImp)<-Chaetocnema_TargetSp 

#### Extract a summary table with selected parameterizations and corresponding AUC statistics from both Maxent and Rf models for each species * CV split combination ####
Chaetocnema_TargetSp_TunedMods_SummaryDf<-pblapply(Chaetocnema_TargetSp,function(sp){
  ### get the summary Df for the tuned Maxent models
  Maxent_TunedMods_Df<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][["AUCstats"]]
  Maxent_TunedMods_Df[,'TunedParam']<-Maxent_TunedMods_Df[,'TunedParam'] %>% gsub("fc.","",.) %>% gsub("rm.","",.)
  colnames(Maxent_TunedMods_Df)<-c("TunedParam","AUCtest","AUCdiff","CVsplit")
  Maxent_TunedMods_Df[,c("AUCtest","AUCdiff")]<-Maxent_TunedMods_Df[,c("AUCtest","AUCdiff")] %>% round(.,2)
  
  ### create a summary Df for the tuned RF models over the Cv splits
  RF_TunedMods_Df<-lapply(1:length(Chaetocnema_TargetSp_TunedRF[[sp]]),function(indx){
    ModDf<-Chaetocnema_TargetSp_TunedRF[[sp]][[indx]][["AUCstats"]]
    ModDf[,c("AUCtest","AUCdiff")]<-ModDf[,c("AUCtest","AUCdiff")] %>% round(.,2)
    
    if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    ModDf_Ref<-data.frame(ModDf,CVsplit=paste0("LOO_",indx))
    }else{ModDf_Ref<-data.frame(ModDf,CVsplit=paste0("Checkerboard_",indx))}
    
    return(ModDf_Ref)}) %>% do.call('rbind',.)
  
  ### extract a summary Df including both algorithms
  colnames(Maxent_TunedMods_Df)[1:3]<-paste0("Maxent_",colnames(Maxent_TunedMods_Df)[1:3])
  colnames(RF_TunedMods_Df)[1:4]<-paste0("RF_",colnames(RF_TunedMods_Df)[1:4])
  
  SummaryDf<-data.frame(Species=rep(sp,length(Chaetocnema_TargetSp_TunedRF[[sp]])),CVsplit=Maxent_TunedMods_Df[,'CVsplit'],Maxent_TunedMods_Df[,1:3],RF_TunedMods_Df[,1:4])
  return(SummaryDf)
}) %>% do.call('rbind',.)

setwd(path_models)
write.csv2(Chaetocnema_TargetSp_TunedMods_SummaryDf,file='Chaetocnema_SouthAfrica_TunedMods_AUC.csv',row.names=F)

### Compute mean and standard deviation of AUCtest and AUCdiff for each species * Cvsplit combination ####
AUC_AvgStats<-mapply(function(sp,algo){
  ### compute mean and std. dev. of AUCtest values for the considered species * algorithm combination
  AUCtest_avg<-Chaetocnema_TargetSp_TunedMods_SummaryDf %>% filter(Species==sp) %>% select(paste0(algo,"_AUCtest")) %>% pull(.) %>% mean(.) %>% round(.,2)
  AUCtest_sd<-Chaetocnema_TargetSp_TunedMods_SummaryDf %>% filter(Species==sp) %>% select(paste0(algo,"_AUCtest")) %>% pull(.) %>% sd(.) %>% round(.,2)
  ### compute mean and std. dev. of AUCdiff values for the considered species * algorithm combination
  AUCdiff_avg<-Chaetocnema_TargetSp_TunedMods_SummaryDf %>% filter(Species==sp) %>% select(paste0(algo,"_AUCdiff")) %>% pull(.) %>% mean(.) %>% round(.,2)
  AUCdiff_sd<-Chaetocnema_TargetSp_TunedMods_SummaryDf %>% filter(Species==sp) %>% select(paste0(algo,"_AUCdiff")) %>% pull(.) %>% sd(.) %>% round(.,2)
  
  ### gather the computed values in a single vector
  AvgStats<-c(AUCtest_avg,AUCtest_sd,AUCdiff_avg,AUCdiff_sd)
  return(AvgStats)},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY = F) %>% do.call('rbind',.) %>% as.data.frame(.)

names(AUC_AvgStats)<-c("AUCtest_Avg","AUCtest_StdDev","AUCdiff_Avg","AUCdiff_StdDev")

AUC_averages<-cbind(Chaetocnema_TargetSp_Algo,AUC_AvgStats)

##### Compare WeightedAvg and WeightedStdDev predictions obtained from tuned Maxent and tuned RF for each species ####
library(RColorBrewer)

if(dir.exists(paste0(path_models,"PredictedSuitability/SummaryPlots"))==F){
  dir.create(paste0(path_models,"PredictedSuitability/SummaryPlots"),recursive = T)}

Suitability_colors<-colorRampPalette(c("gray40","lightgrey","yellow3","yellow","orange","orangered","red2","red4"))(10)
Suitability_breaks<-seq(0,1,0.1)

Suitability_CV_colors<-colorRampPalette(c("greenyellow","chartreuse2","chartreuse4","plum","plum3","plum4"))(10)
Suitability_CV_breaks<-seq(0,0.8,0.08)

Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots<-pblapply(Chaetocnema_TargetSp,function(sp){
  ### occurrences of the target species
  OccurrsDf<-as.data.frame(Chaetocnema_TargetSp_ProjectedOccurr[[sp]])
  names(OccurrsDf)<-c("Species","Easting","Northing")
  
  ### plot weighted average of predictions resulting from Maxent
  WeightedAvg_Maxent_Df<-as.data.frame(as(Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]]$WeightedAvg,"SpatialPixelsDataFrame"))
  names(WeightedAvg_Maxent_Df)<-c("Suitability_WgtAvg","Easting","Northing")
  WeightedAvg_Maxent_Plot<-ggplot()+geom_tile(data=WeightedAvg_Maxent_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtAvg))+
    scale_fill_gradientn(colours=Suitability_colors,breaks=Suitability_breaks,limits=c(0,1),labels=Suitability_breaks)+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_Avg',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  ### plot weighted average of predictions resulting from RF
  WeightedAvg_RF_Df<-as.data.frame(as(Chaetocnema_TargetSp_TunedRF_PredsCurr[[sp]]$WeightedAvg,"SpatialPixelsDataFrame"))
  names(WeightedAvg_RF_Df)<-c("Suitability_WgtAvg","Easting","Northing")
  WeightedAvg_RF_Plot<-ggplot()+geom_tile(data=WeightedAvg_RF_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtAvg))+
    scale_fill_gradientn(colours=Suitability_colors,breaks=Suitability_breaks,limits=c(0,1),labels=Suitability_breaks)+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_Avg',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  ### arrange the two plots in a same panel
  WeightedAvg_Maxent_RF_Plot<-ggarrange(plotlist=list(WeightedAvg_Maxent_Plot,WeightedAvg_RF_Plot),labels=c("Maxent","RF"),nrow=1,ncol=2,align='hv',common.legend = TRUE,legend="right",font.label=list(size=12,face='bold',family='sans',color='black'),label.x = 0.4)
  
  ### plot weighted std. dev. of predictions resulting from Maxent
  WeightedStdDev_Maxent_Df<-as.data.frame(as(Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]]$WeightedStdDev,"SpatialPixelsDataFrame"))
  names(WeightedStdDev_Maxent_Df)<-c("Suitability_WgtStdDev","Easting","Northing")
  WeightedStdDev_Maxent_Plot<-ggplot()+geom_tile(data=WeightedStdDev_Maxent_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtStdDev))+
    scale_fill_gradientn(colours=Suitability_CV_colors,breaks=Suitability_CV_breaks,limits=c(0,0.8),labels=Suitability_CV_breaks)+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_StdDev',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  ### plot weighted std. dev. of predictions resulting from RF
  WeightedStdDev_RF_Df<-as.data.frame(as(Chaetocnema_TargetSp_TunedRF_PredsCurr[[sp]]$WeightedStdDev,"SpatialPixelsDataFrame"))
  names(WeightedStdDev_RF_Df)<-c("Suitability_WgtStdDev","Easting","Northing")
  WeightedStdDev_RF_Plot<-ggplot()+geom_tile(data=WeightedStdDev_RF_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtStdDev))+
    scale_fill_gradientn(colours=Suitability_CV_colors,breaks=Suitability_CV_breaks,limits=c(0,0.8),labels=Suitability_CV_breaks)+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_StdDev',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  ### arrange the two plots in a same panel
  WeightedStdDev_Maxent_RF_Plot<-ggarrange(plotlist=list(WeightedStdDev_Maxent_Plot,WeightedStdDev_RF_Plot),labels=c("Maxent","RF"),nrow=1,ncol=2,align='hv',common.legend = TRUE,legend="right",font.label=list(size=12,face='bold',family='sans',color='black'),label.x = 0.4)
  
  ### plot between-algorithm difference (Maxent - RF) in weighted average of predictions
  WeightedAvg_Diff<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]]$WeightedAvg-Chaetocnema_TargetSp_TunedRF_PredsCurr[[sp]]$WeightedAvg
  WeightedAvg_Diff_Df<-as.data.frame(as(WeightedAvg_Diff,"SpatialPixelsDataFrame"))
  names(WeightedAvg_Diff_Df)<-c("Suitability_WgtAvg_Diff","Easting","Northing")
  WeightedAvg_Diff_Plot<-ggplot()+geom_tile(data=WeightedAvg_Diff_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtAvg_Diff))+
    scale_fill_gradient2(low='red3',high='chartreuse3',mid='gray90')+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_Avg_diff',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  ### plot between-algorithm difference (Maxent - RF) in weighted std. dev. of predictions
  WeightedStdDev_Diff<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]]$WeightedStdDev-Chaetocnema_TargetSp_TunedRF_PredsCurr[[sp]]$WeightedStdDev
  WeightedStdDev_Diff_Df<-as.data.frame(as(WeightedStdDev_Diff,"SpatialPixelsDataFrame"))
  names(WeightedStdDev_Diff_Df)<-c("Suitability_WgtStdDev_Diff","Easting","Northing")
  WeightedStdDev_Diff_Plot<-ggplot()+geom_tile(data=WeightedStdDev_Diff_Df,aes(x=Easting/1000,y=Northing/1000,fill=Suitability_WgtStdDev_Diff))+
    scale_fill_gradient2(low='red3',high='chartreuse3',mid='gray90')+
    geom_point(data=OccurrsDf,aes(x=Easting /1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
    labs(fill='HS_StdDev_diff',y='Northing (km)', x='Easting (km)')+
    theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  
  return(list(WeightedAvg_Maxent_RF_Plot,WeightedStdDev_Maxent_RF_Plot,WeightedAvg_Diff_Plot,WeightedStdDev_Diff_Plot))
})

names(Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots)<-Chaetocnema_TargetSp

for(i in 1:length(Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots)){
  names(Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[i]])<-c("WeightedAvg","WeightedStdDev","WeightedAvg_Diff","WeightedStdDev_Diff")
}

### Export the summary plots
setwd(paste0(path_models,"PredictedSuitability/SummaryPlots"))

### algorithm comparison in Wgt average predictions for species with < 15 occurr
ggarrange(plotlist=lapply(c("brincki","danielssoni","gahani"),function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedAvg}),labels=c("(a)","(b)","(c)"),font.label=list(size=12,family='sans',color='black'),label.x=0.08,ncol=1,nrow=3,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_LowOccurr_WgtAvg_Comparison.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

### algorithm comparison in Wgt average predictions for species with > 15 occurr
ggarrange(plotlist=lapply(c("darwini","natalensis"),function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedAvg}),labels=c("(a)","(b)","(c)"),font.label=list(size=12,face='bold',family='sans',color='black'),label.x=0.08,ncol=1,nrow=2,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_HighOccurr_WgtAvg_Comparison.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

### algorithm comparison in Wgt std. dev. of predictions for species with < 15 occurr
ggarrange(plotlist=lapply(c("brincki","danielssoni","gahani"),function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedStdDev}),labels=c("(a)","(b)","(c)"),font.label=list(size=12,face='bold',family='sans',color='black'),label.x=0.08,ncol=1,nrow=3,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_LowOccurr_WgtStdDev_Comparison.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

### algorithm comparison in Wgt std. dev. of  predictions for species with > 15 occurr
ggarrange(plotlist=lapply(c("darwini","natalensis"),function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedStdDev}),labels=c("(a)","(b)","(c)"),font.label=list(size=12,face='bold',family='sans',color='black'),label.x=0.08,ncol=1,nrow=2,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_HighOccurr_WgtStdDev_Comparison.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

### between-algorithm differences in Wgt average predictions across species
ggarrange(plotlist=lapply(Chaetocnema_TargetSp,function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedAvg_Diff}),labels=Chaetocnema_TargetSp,font.label=list(size=12,face='bold',family='sans',color='black'),label.x=0.2,ncol=2,nrow=3,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_WgtAvg_Maxent_RF_Diff.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

### between-algorithm differences in Wgt std. dev. of predictions across species
ggarrange(plotlist=lapply(Chaetocnema_TargetSp,function(sp){Chaetocnema_TargetSp_WeightedPredsCurr_SummaryPlots[[sp]]$WeightedStdDev_Diff}),labels=Chaetocnema_TargetSp,font.label=list(size=12,face='bold',family='sans',color='black'),label.x=0.2,ncol=2,nrow=3,align='hv',common.legend = T,legend='right')+
  ggsave(filename=paste0("Chaetocnema_TargetSp_WgtStdDev_Maxent_RF_Diff.png"),device=png(type="cairo",units="cm",width=21,height=28,res=600),limitsize = F)

while(dev.cur()>1)dev.off()

##### For each target species, compute weighted avg. and weighted std. dev. of variables importance scores across the spatially blocked train-test splits #####
Chaetocnema_TargetSp_WeightedVarImp_ByAlgo<-pbmapply(function(sp,algo){
  ### retrieve variable importance scores and weight (i.e. AUC on test data) from each train-test split for the considered species
  eval(parse(text=paste0("VarImp_initDf<-Chaetocnema_TargetSp_Tuned",algo,"_VarImp[['",sp,"']]")))
  AUC_weights<-Chaetocnema_TargetSp_TunedMods_SummaryDf %>% filter(Species==sp) %>% select(paste0(algo,"_AUCtest")) %>% pull(.)
  
  ### compute weighted average of importance score
  VarImp_RefDf<-VarImp_initDf %>% mutate(WgtAvg_StdImp=apply(VarImp_initDf,1,weighted.mean,AUC_weights))
  
  ### compute weighted standard deviation of importance scores
  WgtStdDev_Denom<-((length(AUC_weights)-1)/length(AUC_weights))*sum(AUC_weights)
  
  WgtStdDev_StdImp<-lapply(1:nrow(VarImp_initDf),function(indx){
    TargetVar_StdImp<-VarImp_initDf[indx,]
    Wgt_SS<-sapply(1:length(TargetVar_StdImp),function(x){AUC_weights[x]*((TargetVar_StdImp[x]-VarImp_RefDf[indx,'WgtAvg_StdImp'])^2)}) %>% unlist(.)
    WgtStdDev_Numer<-sum(Wgt_SS)
    WgtStdDev<-sqrt(WgtStdDev_Numer/WgtStdDev_Denom)
    return(WgtStdDev)
  }) %>% unlist(.)
  
  ### get a summary dataframe with weighted average and weighted std. dev. of importance scores across the train-test splits for each variable
  WeightedImp<-data.frame(WgtAvg_VarImp=round(VarImp_RefDf[,'WgtAvg_StdImp'],2),WgtStdDev_VarImp=round(WgtStdDev_StdImp,2)) %>% arrange(desc(WgtAvg_VarImp))
  return(WeightedImp)},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY = F)

names(Chaetocnema_TargetSp_WeightedVarImp_ByAlgo)<-paste0(Chaetocnema_TargetSp_Algo$Species,"_",Chaetocnema_TargetSp_Algo$Algo)

### Join in a single dataframe weighted avg. and weighted std. dev. of importance scores from the two algorithms for each target species and export this df as csv
Chaetocnema_TargetSp_TunedMods_WeightedVarImp<-lapply(Chaetocnema_TargetSp,function(sp){
  
  Maxent_Df<-data.frame(Variable=factor(rownames(Chaetocnema_TargetSp_WeightedVarImp_ByAlgo[[paste0(sp,"_Maxent")]]),levels=names(Bioclim_Curr_Sub_SAfr)),Chaetocnema_TargetSp_WeightedVarImp_ByAlgo[[paste0(sp,"_Maxent")]]) %>% arrange(Variable)
  rownames(Maxent_Df)<-NULL
  
  RF_Df<-data.frame(Variable=factor(rownames(Chaetocnema_TargetSp_WeightedVarImp_ByAlgo[[paste0(sp,"_RF")]]),levels=names(Bioclim_Curr_Sub_SAfr)),Chaetocnema_TargetSp_WeightedVarImp_ByAlgo[[paste0(sp,"_RF")]]) %>% arrange(Variable)
  rownames(RF_Df)<-NULL
  
  CoupledAlgo_Df<-cbind(Maxent_Df,RF_Df[,2:3])
  names(CoupledAlgo_Df)<-c("Variable",paste0('Maxent_',names(Maxent_Df)[2:3]),paste0('RF_',names(RF_Df)[2:3]))
  return(CoupledAlgo_Df)
  }) %>% 
  do.call(rbind,.) %>% 
  cbind(Species=rep(Chaetocnema_TargetSp,each=nlayers(Bioclim_Curr_Sub_SAfr)),.)

if(dir.exists(paste0(path_models,"VarImp_RespCurves"))==F){
  dir.create(paste0(path_models,"VarImp_RespCurves"),recursive = T)}

setwd(paste0(path_models,"VarImp_RespCurves"))
write.csv2(Chaetocnema_TargetSp_TunedMods_WeightedVarImp,file="Chaetocnema_TargetSp_VariableImportance.csv",row.names = F)

#### For each species*algorithm combination, draw response curves, from the tuned model selected for each train-test split, of the three variables with the highest weighted average importance scores ###

### For the maxnet Maxent models, the maxnet::response.plot function will be used to extract dataframes representing partial response plots to the target variables
Chaetocnema_TargetSp_TunedMaxent_RespCurvesDfs<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(ENMeval)
  require(maxnet)
  
  ResponseCurves_Dfs<-lapply(1:nrow(Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][["AUCstats"]]),function(indx){
    ### Retrieve the Maxent chosen (sub)optimal parameterization setting for the considered train-test split
    OptimalParam<-Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][["AUCstats"]][indx,"TunedParam"]
    
    ### Get the Maxent model corresponding to the selected parameterization
    if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
      Maxent_OptMod<-eval.models(Chaetocnema_TargetSp_TunedMaxent[[sp]][[indx]][['Maxent_Eval']])[[OptimalParam]]
    }else{Maxent_OptMod<-eval.models(Chaetocnema_TargetSp_TunedMaxent[[sp]][['Maxent_Eval']])[[OptimalParam]]}
    
    ### Retrieve the variables for which response plots should be created (here, top three variables in terms of weighted avg importance score)
    HighContribVars<-Chaetocnema_TargetSp_TunedMods_WeightedVarImp %>% filter(Species==sp) %>% select(Variable,Maxent_WgtAvg_VarImp,Maxent_WgtStdDev_VarImp) %>% top_n(3,wt=Maxent_WgtAvg_VarImp) %>% select(Variable) %>% pull(.)
    ### if more than 3 variables are selected due to equal weighted avg importance score, force to select only the first three
    HighContribVars<-HighContribVars[1:3]
    
    ### Use the response.plot function from the maxnet package to extract partial response to each target variable  
    HighContribVars_RespDf<-lapply(HighContribVars,function(var){
      RP_df<-maxnet::response.plot(mod=Maxent_OptMod,v=as.character(var),type='cloglog',plot=F)
      names(RP_df)<-c("Value","Predicted_Suitability")
      RP_df_Ref<-data.frame(CVsplit=Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][["AUCstats"]][indx,'CVsplit'],Variable=var,RP_df)
      return(RP_df_Ref)
      }) %>% do.call(rbind,.)
      
      return(HighContribVars_RespDf)})
  
  ### Create and extract a summary dataframe reporting modeled partial response to each target variable across all the train-test splits
  ResponseCurves_SummaryDf<-data.frame(Species=sp,ResponseCurves_Dfs %>% do.call(rbind,.))
  
  return(ResponseCurves_SummaryDf)
}) 

names(Chaetocnema_TargetSp_TunedMaxent_RespCurvesDfs)<-Chaetocnema_TargetSp

### For the RF models, dataframes representing partial response to the targte variables will be extracted through an adapted version of the "response.plot" function provided by Dr. Michele Di Musciano
ResponseCurvesDf_MicheleEdit<-function(model, variables_df, n, ...){
  Var_mean<-apply(variables_df, 2, function(x) rep(mean(x), n))
  Pred_2d<-list()
  for(i in 1:ncol(variables_df)){
    Var_mean[,i]<-seq(min(variables_df[,i]), max(variables_df[,i]),  length.out = n)
    if(class(model)=='randomForest'){
      Pred_2d[[i]]<-cbind.data.frame(Pred=predict(model, as.data.frame(Var_mean),type='prob',...)[,2], Value=Var_mean[,i], Var=rep(colnames(Var_mean)[i], n))
    }else{
      Pred_2d[[i]]<-cbind.data.frame(Pred=predict(model, as.data.frame(Var_mean), type="response", ...), Value=Var_mean[,i], Var=rep(colnames(Var_mean)[i], n))}
  }
  Pred_tot2d<-do.call(rbind.data.frame, Pred_2d)
  return(Pred_tot2d)}

Chaetocnema_TargetSp_TunedRF_RespCurvesDfs<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(randomForest)
  
  ModellingVars<-names(Bioclim_Curr_Sub_SAfr)
  
  ResponseCurves_Dfs<-lapply(1:length(Chaetocnema_TargetSp_TunedRF[[sp]]),function(indx){
    ### Retrieve the chosen (sub)optimal RF model
    RF_OptMod<-Chaetocnema_TargetSp_TunedRF[[sp]][[indx]][["RF_OptMod"]]
    
    ### Retrieve the variables for which response plots should be created (here, top three variables in terms of weighted avg importance score)
    HighContribVars<-Chaetocnema_TargetSp_TunedMods_WeightedVarImp %>% filter(Species==sp) %>% select(Variable,RF_WgtAvg_VarImp,RF_WgtStdDev_VarImp) %>% top_n(3,wt=RF_WgtAvg_VarImp) %>% select(Variable) %>% pull(.)
    ### if more than 3 variables are selected due to equal weighted avg importance score, force to select only the first three
    HighContribVars<-HighContribVars[1:3]
    
    ### Retrieve the training data used to fit the optimized random forest model for each train-test split
    Pres_Background_Df<-Chaetocnema_TargetSp_PresBackground_SpDfs[[paste0(sp,"_RF")]]@data
    TrainData<-Pres_Background_Df[Chaetocnema_TargetSp_SpatialBlocking[[paste0(sp,"_RF")]]$folds[[indx]][[1]],]
    
    ### compute partial responses to the highly contributing variables and store response values in a dataframe
    HighContribVars_RespDf<-ResponseCurvesDf_MicheleEdit(model=RF_OptMod,variables_df=TrainData[,ModellingVars],n=100)
    HighContribVars_RespDf_Ref<-data.frame(Variable=HighContribVars_RespDf$Var,Value=HighContribVars_RespDf$Value,Predicted_Suitability=HighContribVars_RespDf$Pred)
    
    if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
      HighContribVars_RespDf_Ref<-cbind(CVsplit=paste0("LOO_",indx),HighContribVars_RespDf_Ref %>% filter(Variable%in%HighContribVars)) 
    }else{HighContribVars_RespDf_Ref<-cbind(CVsplit=paste0("Checkerboard_",indx),HighContribVars_RespDf_Ref %>% filter(Variable%in%HighContribVars))}
    
    return(HighContribVars_RespDf_Ref)
  })
  
  ### create and extract a summary dataframe across the train-test splits
  ResponseCurves_SummaryDf<-data.frame(Species=sp,ResponseCurves_Dfs%>%do.call(rbind,.))
  
  return(ResponseCurves_SummaryDf)
})

names(Chaetocnema_TargetSp_TunedRF_RespCurvesDfs)<-Chaetocnema_TargetSp

#### For each species * algorithm combination, plot response curves of the selected highly contributing variables from each train-test split ####
Chaetocnema_TargetSp_TunedMods_RespCurvesPlots<-pblapply(Chaetocnema_TargetSp,function(sp){
  RespCurvesDf_Maxent<-Chaetocnema_TargetSp_TunedMaxent_RespCurvesDfs[[sp]]
  RespCurvesDf_RF<-Chaetocnema_TargetSp_TunedRF_RespCurvesDfs[[sp]]
  RespCurvesDf_ToPlot<-data.frame(rbind(RespCurvesDf_Maxent,RespCurvesDf_RF),Algorithm=c(rep('Maxent',nrow(RespCurvesDf_Maxent)),rep('RF',nrow(RespCurvesDf_RF))))
  
  RespCurvesDf_ToPlot[,'Value']<-round(RespCurvesDf_ToPlot[,'Value'],2)
  RespCurvesDf_ToPlot[,'Predicted_Suitability']<-round(RespCurvesDf_ToPlot[,'Predicted_Suitability'],2)
  
  if(sp=='brincki'|sp=='danielssoni'|sp=='gahani'){
    RespCurvesDf_ToPlot[,'CVsplit']<-factor(RespCurvesDf_ToPlot[,'CVsplit'],levels=paste0("LOO_",1:as.numeric(Chaetocnema_TargetSp_OccurrNumb %>% filter(Species==sp) %>% select(nOccurr) %>% pull(.))))
  }else{RespCurvesDf_ToPlot[,'CVsplit']<-factor(RespCurvesDf_ToPlot[,'CVsplit'],levels=paste0("Checkerboard_",1:2))}
  
  RespCurvesDf_ToPlot[,'Variable']<-as.character(RespCurvesDf_ToPlot[,'Variable']) %>% gsub("_","",.)
  RespCurvesDf_ToPlot[,'Variable'][RespCurvesDf_ToPlot[,'Variable']%in%c("bio2","bio8","bio9")]<-paste0(RespCurvesDf_ToPlot[,'Variable'][RespCurvesDf_ToPlot[,'Variable']%in%c("bio2","bio8","bio9")]," (°C)")
  RespCurvesDf_ToPlot[,'Variable'][RespCurvesDf_ToPlot[,'Variable']%in%c("bio13","bio14","bio18","bio19")]<-paste0(RespCurvesDf_ToPlot[,'Variable'][RespCurvesDf_ToPlot[,'Variable']%in%c("bio13","bio14","bio18","bio19")]," (mm)")
  RespCurvesDf_ToPlot[,'Variable']<-factor(RespCurvesDf_ToPlot[,'Variable'],levels=c("bio2 (°C)","bio3","bio8 (°C)","bio9 (°C)","bio13 (mm)","bio14 (mm)","bio15","bio18 (mm)","bio19 (mm)"))
  
  Resp_Plot<-ggplot(data=RespCurvesDf_ToPlot,aes(x=Value,y=Predicted_Suitability,group=CVsplit,colour=CVsplit))+
    geom_smooth(se=F)+
    theme_bw()+
    scale_colour_viridis_d(option='E',aesthetics = 'colour')+
    scale_y_continuous(breaks=seq(0,1,by=0.1),limits=c(0,1))+
    facet_wrap(Algorithm~Variable,nrow=2,ncol=3,scales='free')+
    labs(title=paste0(sp,"_TunedModels_ResponseCurves"),y="Predicted suitability", x="",colour='CV split')+
    theme(title=element_text(size=12, face="bold",family='sans'),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans'),axis.text.y=element_text(size=11,family='sans'))
  
  setwd(paste0(path_models,"VarImp_RespCurves"))
  ggsave(filename=paste0(sp,"_ResponseCurves.bmp"),plot=Resp_Plot,device='bmp',units="cm",width=21,height=24,dpi=600,limitsize = F)
  
  return(Resp_Plot)
})

names(Chaetocnema_TargetSp_TunedMods_RespCurvesPlots)<-Chaetocnema_TargetSp

#### From now on, to properly run the code one would need the raster of African vegetation formations and macrogroups 
#### We got this raster from a previous research in which Maurizio Biondi, Paola D'Alessandro some other colleagues digitalized the corresponding maps (having 90 m resolution) provided in Sayre et al. (2013) A new map of standardized terrestrial ecosystems of Africa. African Geographical Review, 1-24
#### As we cannot provide this raster map as Supplementary Materials due to its size, one could use other similar information about vegetation patterns, for instance the Global Land Cover Map provided by the Copernicus service (https://land.copernicus.eu/global/products/), to perform the following analyses

#### Analyse correspondence between climatic suitability for each target species and South Africa vegetation formations ####
#### Load the raster of vegetation formations and macrogroups across Africa in WGS84
Africa_VegMg_WGS84<-raster(paste0(path_root,"\\Africa_VegMacrogroups_labeled\\Africa_IVC_20130316_final_MG.tif"))

### Load the legend of African vegetation formations and macrogroups
library(foreign)
VegMg_legend<-read.dbf(paste0(path_root,"\\Africa_VegMacrogroups_labeled\\Africa_IVC_20130316_final_MG.tif.vat.dbf")) %>% filter(Mapped==1) %>% select(Value,formation,Macrogroup)
names(VegMg_legend)<-c("RsVal","Formation","Macrogroup")

### Get a vector containing the names of the different vegetation formations
VegForm<-VegMg_legend %>% select(Formation) %>% pull(.) %>% unique(.)

### Get a list containing the raster values corresponding to the different macrogroups included in each formation
Africa_VegForm_RsValues<-lapply(VegForm,function(vForm){
  VegForm_Vals<-VegMg_legend %>% filter(Formation==vForm) %>% select(RsVal) %>% pull(.)
  return(VegForm_Vals)
})

names(Africa_VegForm_RsValues)<-VegForm

### Create a reclassification matrix assigning a single raster value to all the macrogroups included in a vegetation formation
### The value 0 (No Data) will remain as is, while the value 800 (corresponding to water) will be turned into 100
VegForm_ReclassificationDf<-cbind(id=c(0,unlist(Africa_VegForm_RsValues) %>% as.numeric(.),800),value=c(0,lapply(1:length(Africa_VegForm_RsValues),function(indx)rep(indx,length(Africa_VegForm_RsValues[[indx]]))) %>% unlist(.) %>% as.numeric(.),100))

### Reclassify the raster of Africa macrogroups to vegetation formations, write the reclassified raster to the hard drive then load it back to get stable connection
Africa_VegForm_WGS84<-raster::reclassify(Africa_VegMg_WGS84,VegForm_ReclassificationDf)
setwd(path_root)
writeRaster(Africa_VegForm_WGS84,filename='AfricaWGS84_VegFormations100m_Sayre2013.tif',format='GTiff')

Africa_VegForm_WGS84<-raster("AfricaWGS84_VegFormations100m_Sayre2013.tif")

### Project the reclassified raster of vegetation formations to Africa sinusoidal projection
### since the projectRaster function delivered writing error for some bins, the projection to the Africa Sinusoidal CRS was accomplished in Qgis v. 3.10.14
AfricaSinu_VegForm<-raster("AfricaSinu_VegFormations100m_Sayre2013.tif")

### Load a stack of the bioclimatic variables used for ENMs fitting, this time spanning the entire Africa
Bioclim_Curr_Africa<-pblapply(Bioclim_names,function(bio){
  ## Load the rasters of worldwide bioclimatic variable at 2.5 arcmin resolution
  Bio_curr_world<-raster(paste0(path_vars_world,"Current\\",grep(paste0(bio,".tif$"),Bioclim_Curr_world_files,value=T)))
  
  ## Crop and mask the loaded raster to the Africa boundaries and then stack it to the Africa sinusoidal coordinate system
  Bio_curr_africa_proj<-projectRaster(mask(crop(Bio_curr_world,extent(Africa_poly_WGS84)),Africa_poly_WGS84),crs='+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs')
  
  ### Extract the cropped and projected raster
  return(Bio_curr_africa_proj)}) %>% stack(.)

names(Bioclim_Curr_Africa)<-Bioclim_names

## For visualization purposes, create an extent object for Africa excluding the South African Prince Edward Island which is located too much southwards
Afr_Ext<-terra::ext(xmin(extent(Africa_poly_WGS84)),xmax(extent(Africa_poly_WGS84)),-36.00,ymax(extent(Africa_poly_WGS84))) 
## Crop the Africa polygon the obtained extent mask, then project it to Africa sinusoidal
Africa_WGS84<-Africa_poly_WGS84 %>% terra::vect(.) %>% terra::crop(.,Afr_Ext) 
Africa_AfrSinu_sf<-as_Spatial(st_as_sf(terra::project(Africa_WGS84,"+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs")))

Bioclim_Curr_Africa<-Bioclim_Curr_Africa %>% crop(.,extent(Africa_AfrSinu_sf)) %>% raster::mask(.,Africa_AfrSinu_sf) %>% stack(.)

#### Aggregate the projected raster of Africa vegetation formations to the resolution of the bioclimatic variables, returning a separate raster file for each formation with percent cover within each aggregated cell ####
if(dir.exists(paste0(path_root,"/Africa_VegMacrogroups_labeled/VegForm_2.5min_PercentCover"))==F){
  dir.create(paste0(path_root,"/Africa_VegMacrogroups_labeled/VegForm_2.5min_PercentCover"),recursive = T)}

plan(multisession,gc=T,workers=nWorkingProcesses)
AfricaSinu_VegForm_PercentCov<-future_map(.x=1:length(Africa_VegForm_RsValues),.f=function(veg_indx){
  VegForm_PercentCov<-raster::aggregate(AfricaSinu_VegForm,fact=res(Bioclim_Curr_Africa)/res(AfricaSinu_VegForm),fun=function(vals,na.rm){
    round((sum(vals==veg_indx,na.rm=na.rm)/length(vals))*100,1)})
  
  ### in case of any computing error returning percent values lower than 0 or higher than 100%, convert these values to 0 and 100 respectively
  VegForm_PercentCov[VegForm_PercentCov<0]<-0
  VegForm_PercentCov[VegForm_PercentCov>100]<-100
  
  ### write the obtained raster to the hard drive, then load it again to get a stable connection
  setwd(paste0(path_root,"/Africa_VegMacrogroups_labeled/VegForm_2.5min_PercentCover"))
  writeRaster(VegForm_PercentCov,filename=paste0(substr(names(Africa_VegForm_RsValues)[veg_indx],start=1,stop=5),"_PercentCover_2.5min.tif"),format='GTiff',overwrite=T)
  
  VegForm_PercentCov_rs<-raster(paste0(substr(names(Africa_VegForm_RsValues)[veg_indx],start=1,stop=5),"_PercentCover_2.5min.tif"))
  return(VegForm_PercentCov_rs)},.progress=T,.options=furrr_options(seed=T))

plan(sequential)

### stack the aggregated raster and resample to dimensions of Africa bioclimatic layers
AfricaSinu_VegForm_PercentCov<-stack(AfricaSinu_VegForm_PercentCov) %>% raster::resample(.,Bioclim_Curr_Africa) %>% mask(.,Bioclim_Curr_Africa)

names(AfricaSinu_VegForm_PercentCov)<-sapply(1:nlayers(AfricaSinu_VegForm_PercentCov),function(indx){paste0("PercentCover_",substr(names(AfricaSinu_VegForm_PercentCov)[indx],start=2,stop=6))})

### Crop the aggregated rasters of vegetation formations percent coverage to the South Africa extent
SAfrSinu_VegForm_PercentCov<-crop(AfricaSinu_VegForm_PercentCov,Bioclim_Curr_Sub_SAfr) %>% raster::resample(.,Bioclim_Curr_Sub_SAfr) %>% mask(.,Bioclim_Curr_Sub_SAfr$bio_2)

### Create a folder to store binarized predicted suitability maps
if(dir.exists(paste0(path_models,"/PredictedSuitability/Binarized"))==F){
  dir.create(paste0(path_models,"/PredictedSuitability/Binarized"),recursive = T)}

path_models_BinaryPreds<-paste0(path_models,"/PredictedSuitability/Binarized")

### Compute the 10th percentile training threshold for each target species * algorithm combination from the WeightedAvg raster
Chaetocnema_TargetSp_WgtAvgSuitab_10thPercentThresh<-pblapply(Chaetocnema_TargetSp,function(sp){
  Occurr_Coord<-Chaetocnema_TargetSp_ProjectedOccurr[[sp]]@coords
  Maxent_10thPerc<-quantile(round(raster::extract(Chaetocnema_TargetSp_TunedMaxent_PredsCurr[[sp]][["WeightedAvg"]],Occurr_Coord),2),0.1,na.rm=T)
  RF_10thPerc<-quantile(round(raster::extract(Chaetocnema_TargetSp_TunedRF_PredsCurr[[sp]][["WeightedAvg"]],Occurr_Coord),2),0.1,na.rm=T)
  Thresholds_10thPerc<-round(c(Maxent_10thPerc,RF_10thPerc),2)
  names(Thresholds_10thPerc)<-paste0("10thPerc_",c("Maxent","RF"))
  return(Thresholds_10thPerc)
}) %>% do.call('rbind',.)

Chaetocnema_TargetSp_WgtAvgSuitab_10thPercentThresh<-data.frame(Species=Chaetocnema_TargetSp,Chaetocnema_TargetSp_WgtAvgSuitab_10thPercentThresh)
colnames(Chaetocnema_TargetSp_WgtAvgSuitab_10thPercentThresh)<-c("Species","10thPerc_Maxent","10thPerc_RF")

#### Binarize, for each species * algorithm combination, the WgtAvg raster using either the 10th percentile threshold or a threshold corresponding to HS >= 0.8 ####
Chaetocnema_TargetSp_BinarizedPreds<-pbmapply(function(sp,algo){
  ### retrieve the raster of across-split weighted average predictions
  eval(parse(text=paste0("Preds_WgtAvg<-Chaetocnema_TargetSp_Tuned",algo,"_PredsCurr[['",sp,"']][['WeightedAvg']]")))
  ### retrieve the 10th percentile training threshold
  Thersh_10thPerc<-Chaetocnema_TargetSp_WgtAvgSuitab_10thPercentThresh %>% filter(Species==sp) %>% select(paste0("10thPerc_",algo)) %>% pull(.)
  
  ### set a reclassification matrix for each chosen threshold
  Rcl_10thPerc<-data.frame(from=c(0,Thersh_10thPerc),to=c(Thersh_10thPerc-0.0000001,1),values=c(0,1))
  Rcl_08<-data.frame(from=c(0,0.8),to=c(0.8-0.0000001,1),values=c(0,1))
  
  ### reclassify the weighted average raster based on the two thresholds and write the obtained binarized to the hard drive
  setwd(path_models_BinaryPreds)
  
  raster::reclassify(Preds_WgtAvg,Rcl_10thPerc,filename=paste0(sp,"_",algo,"_10thPercThresh_Binarized.tif"),format='GTiff',overwrite=T)
  
  raster::reclassify(Preds_WgtAvg,Rcl_08,filename=paste0(sp,"_",algo,"_08Thresh_Binarized.tif"),format='GTiff',overwrite=T)
  
  ### load the reclassified rasters to get stable connections
  BinarizedPreds_10thPercThresh<-raster(paste0(path_models_BinaryPreds,"/",sp,"_",algo,"_10thPercThresh_Binarized.tif"))
  BinarizedPreds_085Thresh<-raster(paste0(path_models_BinaryPreds,"/",sp,"_",algo,"_08Thresh_Binarized.tif"))
  
  BinarizedPreds<-stack(BinarizedPreds_10thPercThresh,BinarizedPreds_085Thresh)
  return(BinarizedPreds)
},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY = F)

names(Chaetocnema_TargetSp_BinarizedPreds)<-paste0(Chaetocnema_TargetSp_Algo$Species,"_",Chaetocnema_TargetSp_Algo$Algo)

for(i in 1:length(Chaetocnema_TargetSp_BinarizedPreds)){
  names(Chaetocnema_TargetSp_BinarizedPreds[[i]])<-c("Binarized_10thPerc","Binarized_08Thresh")
}

#### Export summary plots of the binarized predictions for each species * algorithm * threshold combination
Chaetocnema_TargetSp_BinarizedPreds_Plots<-lapply(Chaetocnema_TargetSp,function(sp){
  #### stack the binarized predictions from the four algorithm * threshold combinations, then convert to Df
  BinarizedPreds_Df<-as.data.frame(as(stack(Chaetocnema_TargetSp_BinarizedPreds[grep(sp,names(Chaetocnema_TargetSp_BinarizedPreds))]),"SpatialPixelsDataFrame"))
  BinarizedPreds_Df_Ref<-BinarizedPreds_Df[,c(3,4,1,2,5,6)]
  colnames(BinarizedPreds_Df_Ref)<-c("Maxent_10thPerc","Maxent_HS_0.8","RF_10thPerc","RF_HS_0.8","Easting","Northing")
  
  ### get the occurrences of the target species to overlay them on the binarized predictions
  OccurrsDf<-as.data.frame(Chaetocnema_TargetSp_ProjectedOccurr[[sp]])
  names(OccurrsDf)<-c("Species","Easting","Northing")
  
  ### plot in a single panel the binarized predictions resulting from the four combinations, the export it
  BinaryPreds_Plots<-lapply(1:4,function(indx){
    DataToPlot<-BinarizedPreds_Df_Ref[,c(indx,5,6)]
    colnames(DataToPlot)<-c("Suitable","Easting","Northing")
    DataToPlot$Suitable_Named<-"Unsuitable"
    DataToPlot$Suitable_Named[which(DataToPlot$Suitable==1)]<-"Suitable"
    ggplot()+geom_tile(data=DataToPlot,aes(x=Easting/1000,y=Northing/1000,fill=as.factor(Suitable_Named)))+
      scale_fill_manual(values=c("chartreuse3","gray60"))+
      geom_point(data=OccurrsDf,aes(x=Easting/1000,y=Northing/1000),colour='black',size=1.8,shape=6,stroke=1.1)+
      labs(fill='Binarized suitability',y='Northing (km)', x='Easting (km)')+
      theme(panel.background=element_rect(fill='white'),panel.grid.major.x=element_blank(),panel.grid.minor.x=element_blank(),panel.grid.major.y=element_blank(),panel.grid.minor.y=element_blank(),legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))
  })
  
  BinaryPreds_Fullplot<-ggarrange(plotlist=BinaryPreds_Plots,ncol=2,nrow=2,align='hv',common.legend =T,legend="bottom",labels=colnames(BinarizedPreds_Df_Ref)[1:4],font.label=list(size=11,face='bold',color='black'),label.x = 0.25)
  
  ggsave(plot=BinaryPreds_Fullplot,filename=paste0(sp,"_BinarizedPredictions.bmp"),device='bmp',units="cm",width=24,height=24,dpi=600,limitsize = F)
  
  return(BinaryPreds_Fullplot)
})

while(dev.cur()>1)dev.off()

names(Chaetocnema_TargetSp_BinarizedPreds_Plots)<-Chaetocnema_TargetSp

#### Analyze percent cover of the different vegetation formations within the binarized climatically suitable pixels ####
Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov<-pbmapply(function(sp,algo){
  ### retrieve the binarized rasters of climatically suitable areas derived from the 10th percentile and HS>=0.8 thresholds and convert them to dataframes
  DfBin_10thPerc<-as.data.frame(as(Chaetocnema_TargetSp_BinarizedPreds[[paste0(sp,"_",algo)]][["Binarized_10thPerc"]],"SpatialPixelsDataFrame")) %>% drop_na(.)
  colnames(DfBin_10thPerc)<-c("Suitable","Easting","Northing")
  DfBin_08Thresh<-as.data.frame(as(Chaetocnema_TargetSp_BinarizedPreds[[paste0(sp,"_",algo)]][["Binarized_08Thresh"]],"SpatialPixelsDataFrame")) %>% drop_na(.)
  colnames(DfBin_08Thresh)<-colnames(DfBin_10thPerc)
  
  ### for each of the two binarized rasters, get percent cover of the different vegetation formations at cells corresponding to climatically suitable areas
  Bin_10thPerc_SuitableClim_VegForms<-lapply(1:nlayers(SAfrSinu_VegForm_PercentCov),function(indx){
    SuitablePixels_VegCover<-raster::extract(SAfrSinu_VegForm_PercentCov[[indx]],DfBin_10thPerc %>% filter(Suitable==1) %>% select(Easting,Northing)) %>% round(.,2)
    VegForm_Code<-rep(substr(names(SAfrSinu_VegForm_PercentCov)[indx],start=2,stop=6),length(SuitablePixels_VegCover))
    Df<-cbind(VegForm_Code,SuitablePixels_VegCover)
    colnames(Df)<-c("Vegetation_Form","Percent_Cover")
    return(Df)}) %>% do.call('rbind',.)
  
  Bin_08Thresh_SuitableClim_VegForms<-lapply(1:nlayers(SAfrSinu_VegForm_PercentCov),function(indx){
    SuitablePixels_VegCover<-raster::extract(SAfrSinu_VegForm_PercentCov[[indx]],DfBin_08Thresh %>% filter(Suitable==1) %>% select(Easting,Northing)) %>% round(.,2)
    VegForm_Code<-rep(substr(names(SAfrSinu_VegForm_PercentCov)[indx],start=2,stop=6),length(SuitablePixels_VegCover))
    Df<-cbind(VegForm_Code,SuitablePixels_VegCover)
    colnames(Df)<-c("Vegetation_Form","Percent_Cover")
    return(Df)}) %>% do.call('rbind',.)
  
  SuitableClim_VegForms_FinalDf<-rbind(Bin_10thPerc_SuitableClim_VegForms,Bin_08Thresh_SuitableClim_VegForms) %>% as.data.frame(.) %>%  mutate(Threshold=c(rep("Suitability_10thPercentile",nrow(Bin_10thPerc_SuitableClim_VegForms)),rep("Suitability_0.8",nrow(Bin_08Thresh_SuitableClim_VegForms))))
  return(SuitableClim_VegForms_FinalDf)
},Chaetocnema_TargetSp_Algo$Species,Chaetocnema_TargetSp_Algo$Algo,SIMPLIFY = F)

names(Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov)<-paste0(Chaetocnema_TargetSp_Algo$Species,"_",Chaetocnema_TargetSp_Algo$Algo)

Chaetocnema_LowOccurrSp_VegFormsCover_Df<-lapply(c("brincki","danielssoni","gahani"),function(sp){
  Dfs_Ls<-Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov[grep(sp,names(Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov))]
  Df_final<-lapply(1:2,function(indx){
    Df_ref<-Dfs_Ls[[indx]] %>% mutate(Species=sp,Algorithm=ModellingAlgo[indx])
    return(Df_ref)}) %>% do.call('rbind',.)
  return(Df_final)
}) %>% do.call('rbind',.)

Chaetocnema_HighOccurrSp_VegFormsCover_Df<-lapply(c("darwini","natalensis"),function(sp){
  Dfs_Ls<-Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov[grep(sp,names(Chaetocnema_TargetSp_BinarizedPreds_VegFormsCov))]
  Df_final<-lapply(1:2,function(indx){
    Df_ref<-Dfs_Ls[[indx]] %>% mutate(Species=sp,Algorithm=ModellingAlgo[indx])
    return(Df_ref)}) %>% do.call('rbind',.)
  return(Df_final)
}) %>% do.call('rbind',.)

#### Extract summary figures of association between climatic suitability and vegetation formations for the five target species ####
Chaetocnema_LowOccurrSp_Suitability_VegFormsCover_Plots<-lapply(c("brincki","danielssoni","gahani"),function(sp){
  Plot<-ggplot(Chaetocnema_LowOccurrSp_VegFormsCover_Df %>% filter(Species==sp),aes(y=as.numeric(Percent_Cover),x=Vegetation_Form))+
    geom_boxplot(aes(colour=Threshold),outlier.color='gray70',outlier.size=0.4,outlier.alpha = 0.2)+
    scale_colour_viridis_d(option='D')+
    facet_wrap(~Algorithm,nrow=1,ncol=2)+
    labs(y="Suitable cells cover (%)", x="Vegetation formations",colour='Binarization threshold')+
    theme_bw()+
    theme(legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))+
    scale_y_continuous(breaks=seq(0,100,by=10))})

ggarrange(plotlist = Chaetocnema_LowOccurrSp_Suitability_VegFormsCover_Plots,labels=c("(a)","(b)","(c)"),font.label=list(size=12,family='sans',color='black'),ncol=1,nrow=3,align='hv',common.legend = T,legend='bottom')+
  ggsave(filename="Chaetocnema_LowOccurrSp_Suitability_VegForms.bmp",device='bmp',units="cm",width=21,height=24,dpi=600,limitsize = F)

Chaetocnema_HighOccurrSp_Suitability_VegFormsCover_Plots<-lapply(c("darwini","natalensis"),function(sp){
  Plot<-ggplot(Chaetocnema_HighOccurrSp_VegFormsCover_Df %>% filter(Species==sp),aes(y=as.numeric(Percent_Cover),x=Vegetation_Form))+
    geom_boxplot(aes(colour=Threshold),outlier.color='gray70',outlier.size=0.4,outlier.alpha = 0.2)+
    scale_colour_viridis_d(option='D')+
    facet_wrap(~Algorithm,nrow=1,ncol=2)+
    labs(y="Suitable cells cover (%)", x="Vegetation formations",colour='Binarization threshold')+
    theme_bw()+
    theme(legend.title=element_text(size=12, face="bold",family='sans'),legend.text=element_text(size=12,family='sans'),axis.title.x=element_text(size=12,family='sans'),axis.title.y=element_text(size=12,family='sans'),axis.text.x=element_text(size=11,family='sans',angle=-60,vjust=0.2),axis.text.y=element_text(size=11,family='sans'))+
    scale_y_continuous(breaks=seq(0,100,by=10))})

ggarrange(plotlist = Chaetocnema_HighOccurrSp_Suitability_VegFormsCover_Plots,labels=c("(a)","(b)"),font.label=list(size=12,family='sans',color='black'),ncol=1,nrow=2,align='hv',common.legend = T,legend='bottom')+
  ggsave(filename="Chaetocnema_HighOccurrSp_Suitability_VegForms.bmp",device='bmp',units="cm",width=21,height=24,dpi=600,limitsize = F)

while(dev.cur()>1)dev.off()

###### Implement the PCA-env approach (see Broenniman et al. 2012) to compute niche overlap and perform niche equivalency/similarity tests between each pair of target species ##########
dir.create(paste0(path_root,"\\NicheOverlap"))
path_ecospat<-paste0(path_root,"\\NicheOverlap")
library(ecospat)

### PCA-Env across the study region considering bioclimatic variables + percent cover of vegetation formations ####
SAfr_Bioclim_VegForms<-stack(Bioclim_Curr_SAfr,SAfrSinu_VegForm_PercentCov)

### randomly sample 10000 points across the study area for each variable
VarsOnStudyArea<-as.data.frame(raster::sampleRandom(SAfr_Bioclim_VegForms,10000,xy=T,ext=extent(SAfr_Bioclim_VegForms))) %>% drop_na(.)
names(VarsOnStudyArea)<-c('Easting','Northing',gsub("_","",Bioclim_names),paste0("PercentCov_",substr(names(SAfrSinu_VegForm_PercentCov),2,6)))

#### create a buffer of 120 km (i.e. ~ 1 degree) around occurrence points of each target species, then randomly draw 500 points from these buffer to sample the environmental conditions within 'available areas' for the considered species ####
Chaetocnema_SAfr_BckGrnd<-pblapply(Chaetocnema_TargetSp,function(sp){
  require(sf)
  Occs_projected.sf<-sf::st_as_sf(Chaetocnema_TargetSp_ProjectedOccurr[[sp]], coords = c("Easting","Northing"), crs = projection(SAfr_Bioclim_VegForms))
  Buff_1Degree<-sf::st_buffer(Occs_projected.sf, dist = 120000) %>% sf::st_union() %>% sf::st_sf()
  BgPoints<-st_coordinates(st_sample(Buff_1Degree,size=500))
  BgPoints_VarsVals<-raster::extract(SAfr_Bioclim_VegForms,BgPoints) %>% as.data.frame(.) %>% drop_na(.)
  names(BgPoints_VarsVals)<-c(gsub("_","",Bioclim_names),paste0("PercentCov_",substr(names(SAfrSinu_VegForm_PercentCov),2,6)))
  return(list(Buff_1Degree,BgPoints_VarsVals))
})

names(Chaetocnema_SAfr_BckGrnd)<-Chaetocnema_TargetSp

for(i in 1:length(Chaetocnema_SAfr_BckGrnd)){names(Chaetocnema_SAfr_BckGrnd[[i]])<-c("Buff_1Degree","BgPoints_VarsVals")}

Chaetocnema_SAfr_VarsOnOccurr<-lapply(Chaetocnema_TargetSp,function(sp){
  Occurr_VarsVals<-raster::extract(SAfr_Bioclim_VegForms,Chaetocnema_TargetSp_ProjectedOccurr[[sp]]@coords) %>% as.data.frame(.) %>% drop_na(.)
  names(Occurr_VarsVals)<-c(gsub("_","",Bioclim_names),paste0("PercentCov_",substr(names(SAfrSinu_VegForm_PercentCov),2,6)))
  return(Occurr_VarsVals)})

names(Chaetocnema_SAfr_VarsOnOccurr)<-Chaetocnema_TargetSp

### create a Df with values of the variables extracted from the occurrence points of the five species, and from the random points previously drawn from the entire study area
Vars_Occurr_StudyArea<-rbind(Chaetocnema_SAfr_VarsOnOccurr %>% do.call('rbind',.),VarsOnStudyArea[,3:ncol(VarsOnStudyArea)]) %>% data.frame(PointSource=c(sapply(Chaetocnema_TargetSp,function(sp){rep(sp,nrow(Chaetocnema_SAfr_VarsOnOccurr[[sp]]))}) %>% unlist(.,use.names = F),rep('Background_StudyArea',nrow(VarsOnStudyArea))),.)
rownames(Vars_Occurr_StudyArea)<-NULL

#### reduce the set of bioclimatic variables and vegetation coverages to uncorrelated Principal Components through the PCA-Env procedure ####
# build the vector of weights to be included in the PCA
PCAenv_weight<-c(rep(0,nrow(Chaetocnema_SAfr_VarsOnOccurr %>% do.call('rbind',.))),rep(1,nrow(VarsOnStudyArea)))
# proceed to the PCA and plot the axes
PCA_StudyArea<-dudi.pca(Vars_Occurr_StudyArea[,-1],row.w=PCAenv_weight,center = T, scale = T,scannf=F,nf=3)

# PCA scores for the whole study area
StudyArea_PCAscores <-PCA_StudyArea$li

# Compute the density grids (i.e. niche occupancy) for the target species in the 2D environmental space resulting from the first two axes of the PCA
### I had to slightly modify the ecospat.grid.clim() function from the ecospat package, as it got trouble with possible NAs within the glob1r raster (which defines a kernel-based "density of occurrence" of the background points within the 2D PCA-derived environmental space) generated along the function workflow
### You may try the default ecospat.grid.clim() function; if it does not work for you either, contact me at francesco.cerasoli@univaq.it and I'll send you my modified version of this function
source(paste0(path_ecospat,"\\Grid_clim_dyn_fun_modified.R"))   

Chaetocnema_SAfr_NicheOccupancy<-pblapply(Chaetocnema_TargetSp,function(sp){
  # create a dataframe with the values of the predictors on both presences and background points 
  Vars_Occurr_Bg_Vals<-rbind(Chaetocnema_SAfr_VarsOnOccurr[[sp]],Chaetocnema_SAfr_BckGrnd[[sp]][['BgPoints_VarsVals']])
  
  # compute the PCA scores on occurrences and background points
  Occurr_PCAscores<-suprow(PCA_StudyArea,Vars_Occurr_Bg_Vals[1:nrow(Chaetocnema_SAfr_VarsOnOccurr[[sp]]),])$li
  Bg_PCAscores<-suprow(PCA_StudyArea,Vars_Occurr_Bg_Vals[(nrow(Chaetocnema_SAfr_VarsOnOccurr[[sp]])+1):nrow(Vars_Occurr_Bg_Vals),])$li
  
  # compute the density grid in environmental space using the modified grid.clim.dyn function
  DensityGrid<-grid.clim.dyn(glob=StudyArea_PCAscores[,1:2] %>% drop_na(.),
                             glob1=Bg_PCAscores[,1:2] %>% drop_na(.),
                             sp=Occurr_PCAscores[,1:2] %>% drop_na(.),
                             R=1000)
  return(DensityGrid)})

names(Chaetocnema_SAfr_NicheOccupancy)<-Chaetocnema_TargetSp

### plot variables (bioclimatic + vegetation cover) contributions along the first two axes of the PCA-env
library(factoextra)

PCA_Env_VariableContrib<-fviz_pca_var(PCA_StudyArea,axes=c(1,2),geom.var=c("arrow","text"),repel=T,select.var=list(name=c(gsub("_","",ModellingVars_SAfr),gsub("PercentCover_","Form_",names(SAfrSinu_VegForm_PercentCov)))),col.var="contrib",alpha.var="contrib",gradient.cols=RColorBrewer::brewer.pal(10,'PuOr'),circlesize=0.3,col.circle="red")+
  labs(col="Contribution")+
  guides(alpha='none')+
  labs(x="PrinComp1 (25.3 %)")+
  labs(y="PrinComp2 (15.9 %)")

### plot the niche density grids obtained for the five species on the same PCA-derived 2D
source(paste0(path_ecospat,"\\Ecospat_plot_niche_custom.R"))  ### here I modified the 'ecospat.plot.niche()' function to customize graphical parameters so as to plainly show the density of occurrence (in the PCA-derived 2D space) of all the target species on a same panel

Chaetocnema_SAfr_PCAenv_DensityGrids<-pblapply(Chaetocnema_TargetSp[c(5,3,4,2,1)],function(sp){
  require(oceanmap)
  if(sp=='natalensis'){
    ### for natalensis, use a green-to-purple color scale
    colo=hcl.colors(100, palette='PRGn',alpha=0.9)
    ecospat.plot.niche.custom(Chaetocnema_SAfr_NicheOccupancy[[sp]],name.axis1="PrinComp1 (25.3 %)",name.axis2="PrinComp2 (15.9 %)",density_colPal=colo,cor=T)
    set.colorbar(cbx=c(-9,-8.5),cby=c(5.3,7.7),pal=colo,ticks=c(0,0.5,1),oticks="l",cb.title="natalensis",gradient="horizontal")
  }else if(sp=='darwini'){
    ### for darwini, use a yellow-orange-red color scale
    par(new=T)
    colo=hcl.colors(100,palette='YlOrRd',alpha=0.85)
    ecospat.plot.niche.custom(Chaetocnema_SAfr_NicheOccupancy[[sp]],name.axis1="PrinComp1 (25.3 %)",name.axis2="PrinComp2 (15.9 %)",density_colPal=colo,cor=T)
    set.colorbar(cbx=c(-6.5,-6),cby=c(5.3,7.7),pal=colo,ticks=c(0,0.5,1),oticks="l",cb.title="darwini",gradient="horizontal")
  }else if(sp=='gahani'){
    ### for gahani, use the viridis colour scale
    par(new=T)
    colo=hcl.colors(100, palette='Viridis',alpha=0.8)
    ecospat.plot.niche.custom(Chaetocnema_SAfr_NicheOccupancy[[sp]],name.axis1="PrinComp1 (25.3 %)",name.axis2="PrinComp2 (15.9 %)",density_colPal=colo,cor=T)
    set.colorbar(cbx=c(-4,-3.5),cby=c(5.3,7.7),pal=colo,ticks=c(0,0.5,1),oticks="l",cb.title="gahani",gradient="horizontal")
  }else if(sp=='danielssoni'){
    ### for danielssoni, use the cividis color scale
    par(new=T)
    colo=hcl.colors(100, palette='Cividis',alpha=0.75)
    ecospat.plot.niche.custom(Chaetocnema_SAfr_NicheOccupancy[[sp]],name.axis1="PrinComp1 (25.3 %)",name.axis2="PrinComp2 (15.9 %)",density_colPal=colo,cor=T)
    set.colorbar(cbx=c(-1.5,-1),cby=c(5.3,7.7),pal=colo,ticks=c(0,0.5,1),oticks="l",cb.title="danielssoni",gradient="horizontal")
  }else{
    ### for brincki, use a yellow-to-blue rainbow color scale
    par(new=T)
    colo=rainbow(100, start=rgb2hsv(col2rgb('yellow'))[1], end=rgb2hsv(col2rgb('blue'))[1],alpha=0.9)
    ecospat.plot.niche.custom(Chaetocnema_SAfr_NicheOccupancy[[sp]],name.axis1="PrinComp1 (25.3 %)",name.axis2="PrinComp2 (15.9 %)",density_colPal=colo,cor=T)
    set.colorbar(cbx=c(1,1.5),cby=c(5.3,7.7),pal=colo,ticks=c(0,0.5,1),oticks="l",cb.title="brincki",gradient="horizontal")
  }
})
while(dev.cur()>1)dev.off()

### Compute niche overlap, based on Schoener's D metric, for each pair of species ###
Chaetocnema_SAfr_SpPairs<-expand.grid(Species_1=Chaetocnema_TargetSp,Species_2=Chaetocnema_TargetSp) 
Chaetocnema_SAfr_SpPairs<-Chaetocnema_SAfr_SpPairs[-c(1,6,7,11,12,13,16,17,18,19,21:25),]

Chaetocnema_SAfr_NicheOverlap<-pbmapply(function(sp1,sp2){
  Overlap=ecospat.niche.overlap(Chaetocnema_SAfr_NicheOccupancy[[sp1]],Chaetocnema_SAfr_NicheOccupancy[[sp2]],cor=T)
  ### get only Schoener's D as measure of niche overlap
  return(Overlap[[1]])
},Chaetocnema_SAfr_SpPairs$Species_1,Chaetocnema_SAfr_SpPairs$Species_2,SIMPLIFY = F) %>% unlist(.) %>% as.vector(.) %>% round(.,2)

Chaetocnema_SAfr_NicheOverlap_Df<-data.frame(Chaetocnema_SAfr_SpPairs,Schoener_D=Chaetocnema_SAfr_NicheOverlap)

### Make niche similarity tests, for both the niche divergence and niche conservatism hypotheses, for each pair of species
Chaetocnema_SAfr_NicheTests<-pbmapply(function(sp1,sp2){
  ### similarity tests using the ecospat.niche.similarity.test function
  SimTest_Div<-ecospat.niche.similarity.test(Chaetocnema_SAfr_NicheOccupancy[[sp1]],Chaetocnema_SAfr_NicheOccupancy[[sp2]],rep=100,alternative="lower",rand.type = 1)
  SimTest_Cons<-ecospat.niche.similarity.test(Chaetocnema_SAfr_NicheOccupancy[[sp1]],Chaetocnema_SAfr_NicheOccupancy[[sp2]],rep=100,alternative="greater",rand.type = 1)
  
  ### summary Df
  NicheTests_Df<-data.frame(SimTest_Divergence=round(SimTest_Div[["p.D"]],3),SimTest_Conservatism=round(SimTest_Cons[["p.D"]],3))
  return(NicheTests_Df)},Chaetocnema_SAfr_SpPairs$Species_1,Chaetocnema_SAfr_SpPairs$Species_2,SIMPLIFY = F) %>% do.call('rbind',.)

### Export a summary Df storing Schoener's d and p-values from niche similarity tests
Chaetocnema_SAfr_NicheOverlap_NicheTests<-cbind(Chaetocnema_SAfr_NicheOverlap_Df,Chaetocnema_SAfr_NicheTests) %>% arrange(desc(Schoener_D))
setwd(path_ecospat)
write.csv2(Chaetocnema_SAfr_NicheOverlap_NicheTests,file = 'Chaetocnema_SAfr_PCAenv_NicheOverlap.csv',row.names =F)

#### End of the analyses ####
